"""
Reads and writes .astrom files.
"""
import math

__author__ = "David Rusk <drusk@uvic.ca>"
import os
import re

from astropy import units
from astropy.coordinates import SkyCoord
from astropy.units import Quantity
from astropy.time import TimeDelta, Time
import util

from .gui import logger
from . import storage

DATASET_ROOT = storage.DBIMAGES

# Images from CCDs < 18 have their coordinates flipped
MAX_INVERTED_CCD = 17
INVERTED_CCDS = range(0, 18)
INVERTED_CCDS.append(36)
INVERTED_CCDS.append(37)

HEADER_LINE_LENGTH = 80

FAKE_PREFIX = "fk"

OBS_LIST_PATTERN = "#\s+(?P<rawname>(?P<fk>%s)?(?P<expnum>\d{6,7})(?P<ftype>[ops])(?P<ccdnum>\d+))" % FAKE_PREFIX

STATIONARY_LIST_PATTERN = "(?P<rawname>(?P<fk>fk)?(?P<expnum>\d{6,7})(?P<ftype>[ops])).vetting"

# Observation header keys
MOPVERSION = "MOPversion"

# NOTE: MJD_OBS_CENTER is actually MJD-OBS-CENTER in the .astrom files, but
# dashes aren't valid as regex names so I use underscores
MJD_OBS_CENTER = "MJD_OBS_CENTER"
EXPTIME = "EXPTIME"
THRES = "THRES"
FWHM = "FWHM"
MAXCOUNT = "MAXCOUNT"
CRVAL1 = "CRVAL1"
CRVAL2 = "CRVAL2"
EXPNUM = "EXPNUM"
SCALE = "SCALE"
CHIP = "CHIP"
CRPIX1 = "CRPIX1"
CRPIX2 = "CRPIX2"
NAX1 = "NAX1"
NAX2 = "NAX2"
DETECTOR = "DETECTOR"
PHADU = "PHADU"
RDNOIS = "RDNOIS"

# System header keys
RMIN = "RMIN"
RMAX = "RMAX"
ANGLE = "ANGLE"
AWIDTH = "AWIDTH"


def parse(filename):
    return AstromParser().parse(filename)


def parse_sources(filename):
    return parse(filename).get_sources()


class AstromFormatError(Exception):
    """Base class for errors in working with Astrom files."""


class AstromParser(object):
    """
    Parses a .astrom file (our own format) which specifies exposure numbers,
    identified point sources, their x, y location, source readings for
    potential moving objects, etc.
    """

    def __init__(self):
        """Creates the parser"""

        # Set up the regexes need to parse each section of the .astrom file

        self.obs_list_regex = re.compile(OBS_LIST_PATTERN)

        self.obs_header_regex = re.compile(
            "##\s+MOPversion\s+#\s+"
            "(?P<MOPversion>\d+\.[\d\w]+)\s+"
            "##\s+MJD-OBS-CENTER\s+EXPTIME\s+THRES\s+FWHM\s+MAXCOUNT\s+CRVAL1\s+CRVAL2\s+EXPNUM\s+"
            "#\s+(?P<MJD_OBS_CENTER>\d{4} \d{2} \d+\.\d+)\s+"
            "(?P<EXPTIME>\d+\.\d+)\s+"
            "(?P<THRES>\d+\.\d+)\s+"
            "(?P<FWHM>\d+\.\d+)\s+"
            "(?P<MAXCOUNT>\d+\.\d+)\s+"
            "(?P<CRVAL1>-?\d+\.\d+)\s+"
            "(?P<CRVAL2>-?\d+\.\d+)\s+"
            "(?P<EXPNUM>\d+)\s+"
            "##\s+SCALE\s+CHIP\s+CRPIX1\s+CRPIX2\s+NAX1\s+NAX2\s+DETECTOR\s+PHADU\s+RDNOIS\s+#\s+"
            "(?P<SCALE>\d+\.\d+)\s+"
            "(?P<CHIP>\d+)\s+"
            "(?P<CRPIX1>-?\d+\.\d+)\s+"
            "(?P<CRPIX2>-?\d+\.\d+)\s+"
            "(?P<NAX1>\d+)\s+"
            "(?P<NAX2>\d+)\s+"
            "(?P<DETECTOR>\w+)\s+"
            "(?P<PHADU>\d+\.\d+)\s+"
            "(?P<RDNOIS>\d+\.\d+)"
        )

        self.sys_header_regex = re.compile(
            "##\s+RMIN\s+RMAX\s+ANGLE\s+AWIDTH\s+#\s+"
            "(?P<RMIN>\d+\.\d+)\s+"
            "(?P<RMAX>\d+\.\d+)\s+"
            "(?P<ANGLE>-?\d+\.\d+)\s+"
            "(?P<AWIDTH>\d+\.\d+)"
        )

        self.source_list_reg = re.compile(
            "##\s+X\s+Y\s+X_0\s+Y_0\s+R.A.\s+DEC\s+(.*)",
            re.DOTALL
        )
        # Should we only load the discovery images during Candidate vetting?
        self.discovery_only = False

    def _parse_observation_list(self, filestr):
        matches = self.obs_list_regex.findall(filestr)  # returns list of tuples
        return [Observation.from_parse_data(*match) for match in matches]

    def _parse_observation_headers(self, filestr, observations):
        obsnum = 0
        for match in self.obs_header_regex.finditer(filestr):
            obs = observations[obsnum]
            for header_key, header_val in match.groupdict().iteritems():
                obs.header[header_key] = header_val
            obsnum += 1

        assert obsnum == len(observations), ("Number of observations headers "
                                             "parsed doesn't match length of "
                                             "observation list")

    def _parse_system_header(self, filestr):
        sys_header_match = self.sys_header_regex.search(filestr)

        assert sys_header_match is not None, "Could not parse system header"

        return sys_header_match.groupdict()

    def _parse_source_data(self, file_str, observations):
        source_list_match = self.source_list_reg.search(file_str)

        assert source_list_match is not None, "Could not find the source list"

        raw_source_list = (source_list_match.group(1)).split("\n\n")

        sources = []
        for raw_source in raw_source_list:
            source = []
            source_obs = raw_source.strip().split('\n')
            assert len(source_obs) == len(
                observations), ("Source doesn't have same number of observations"
                                " ({0:d}) as in observations list ({1:d}).".format(len(source_obs), len(observations)))

            x_0 = []
            y_0 = []
            x_ref = None
            y_ref = None
            for i, source_ob in enumerate(source_obs):
                fields = [float(x) for x in source_ob.split()]
                x_0.append(fields[2])
                y_0.append(fields[3])
                if i == 0:
                    x_ref = fields[0]
                    y_ref = fields[1]
                fields.append(x_ref)
                fields.append(y_ref)
                # Find the observation corresponding to this reading
                fields.append(observations[i])

                source.append(SourceReading(*fields))

            # Add an ra/dec reference to the source.
            ref_index = int(math.ceil(len(source) / 2.0)) - 1
            for reading in source:
                assert isinstance(reading, SourceReading)
                reading.reference_sky_coord = source[ref_index].sky_coord

            # determine the smallest cutout that will include the reference coordinate and all the readings
            min_cutout = 30 * units.arcsec
            for reading in source:
                sep = reading.reference_sky_coord.separation(reading.sky_coord)
                if min_cutout < sep:
                    min_cutout = sep
            # Overload the 'uncertainty' criterion to ensure we get a large enough cutout.
            for fields in source:
                assert isinstance(fields, SourceReading)
                fields.uncertainty_ellipse.a = sep/2.5
                fields.uncertainty_ellipse.b = sep/2.5
                fields.uncertainty_ellipse.pa = 0.0 * units.degree

            sources.append(source)

        return sources

    def parse(self, filename):
        """
        Parses a file into an AstromData structure.

        Args:
          filename: str
            The name of the file whose contents will be parsed.

        Returns:
          data: AstromData
            The file contents extracted into a data structure for programmatic
            access.
        """
        filehandle = storage.open_vos_or_local(filename, "rb")
        assert filehandle is not None, "Failed to open file {} ".format(filename)
        filestr = filehandle.read()
        filehandle.close()

        assert filestr is not None, "File contents are None"

        observations = self._parse_observation_list(filestr)

        self._parse_observation_headers(filestr, observations)

        sys_header = self._parse_system_header(filestr)

        sources = self._parse_source_data(filestr, observations)

        return AstromData(observations, sys_header, sources, discovery_only=self.discovery_only)


class StationaryParser(AstromParser):

    def __init__(self, discovery_only=True):
        super(StationaryParser, self).__init__()
        self.observations = []
        self.obs_list_regex = re.compile(STATIONARY_LIST_PATTERN)
        self.discovery_only = discovery_only

    def _parse_observation_list(self, filestr):
        pass

    def _parse_observation_headers(self, filestr, observations):
        """
        This should provide a list of headers that go with the given observation, as read from the input
        file, but for stationary sources we grab those from the storage system.

        @param filestr:  string to scan for observation header info.
        @param observations:  list of observations whose headers will be augmented by the contents in filestr
        @return:

        """
        pass

    def _parse_system_header(self, filestr):
        """
        For stationary catalog check we don't have this information.
        @param filestr: string to match for system header information.
        @return: dict
        """
        return {'RMIN': 0.01, 'RMAX': 0.2, 'ANGLE': 0, 'AWIDTH': 90}

    def _parse_source_data(self, file_str, observations):

        sources = []
        observations = {}
        raw_source_list = file_str.split("\n")
        for raw_source in raw_source_list:
            readings = []
            if not len(raw_source) > 0:
                continue
            parts = raw_source.split()
            if not len(parts) > 5:
                raise AstromFormatError("line in vetting file has too few elements in line: {}".format(raw_source))
            object_id = parts[0]
            ra = float(parts[1])
            dec = float(parts[2])
            file_ids = parts[3:]
            count = 0
            for file_id in file_ids:
                count += 1
                match_groups = re.match("(?P<expnum>\d{6,7})(?P<ftype>[ops])(?P<ccdnum>\d{2})", file_id)
                assert match_groups is not None, "Failed to parse file_id in stationary line"
                expnum = int(match_groups.group('expnum'))
                ftype = match_groups.group('ftype')
                ccd = match_groups.group('ccdnum')
                observation = Observation(expnum, ftype=ftype, ccdnum=ccd)
                source_reading = SourceReading(-1, -1, -1, -1,
                                               ra, dec, -1, -1,
                                               observation,
                                               discovery=count < 4
                                               )
                source_reading.reference_sky_coord = source_reading.sky_coord
                source_reading.object_id = object_id
                readings.append(source_reading)
            sources.append(readings)
        return sources


class BaseAstromWriter(object):
    """
    Provides base functionality for AstromWriters.  Use the subclass for your
    use case.
    """

    def __init__(self, filehandle):
        self.output_file = filehandle

        self._header_written = False

    def get_filename(self):
        return self.output_file.name

    def _write_line(self, line, ljust=True):
        if ljust:
            line = line.ljust(HEADER_LINE_LENGTH)

        self.output_file.write(line + "\n")

    def _write_blank_line(self):
        self._write_line("", ljust=False)

    def _write_observation_list(self, observations):
        for observation in observations:
            self._write_line("# %s" % observation.rawname)

    def _write_observation_headers(self, observations):
        """
        See src/pipematt/step1matt-c
        """
        for observation in observations:
            header = observation.header

            def get_header_vals(header_list):
                header_vals = []
                for key in header_list:
                    val = header[key]

                    if key == MJD_OBS_CENTER:
                        header_vals.append(val)
                    elif key == DETECTOR:
                        header_vals.append(val.ljust(20))
                    else:
                        header_vals.append(float(val))

                return tuple(header_vals)

            self._write_line("## MOPversion")
            self._write_line("#  %s" % header[MOPVERSION])
            self._write_line("## MJD-OBS-CENTER  EXPTIME THRES FWHM  MAXCOUNT CRVAL1     CRVAL2     EXPNUM")
            self._write_line("# %s%8.2f%6.2f%6.2f%9.1f%11.5f%11.5f%9d" % get_header_vals(
                [MJD_OBS_CENTER, EXPTIME, THRES, FWHM, MAXCOUNT, CRVAL1, CRVAL2, EXPNUM]))
            self._write_line("## SCALE CHIP CRPIX1    CRPIX2    NAX1  NAX2   DETECTOR           PHADU RDNOIS")
            self._write_line("# %6.3f%4d%10.2f%10.2f%6d%6d %s%5.2f %5.2f" % get_header_vals(
                [SCALE, CHIP, CRPIX1, CRPIX2, NAX1, NAX2, DETECTOR, PHADU, RDNOIS]))

    def _write_sys_header(self, sys_header):
        """
        See src/pipematt/step3matt-c
        """
        header_vals = [sys_header[RMIN], sys_header[RMAX], sys_header[ANGLE],
                       sys_header[AWIDTH]]
        self._write_line("##     RMIN    RMAX   ANGLE   AWIDTH")
        self._write_line("# %8.1f%8.1f%8.1f%8.1f" % tuple(map(float, header_vals)))

    def _write_source_data(self, sources):
        """
        See src/jjk/measure3
        """
        for i, source in enumerate(sources):
            self._write_source(source)

    def _write_source(self, source):
        self._write_blank_line()

        for reading in source.get_readings():
            self._write_line(" %8.2f %8.2f %8.2f %8.2f %12.7f %12.7f" % (
                reading.x, reading.y, reading.x0, reading.y0, reading.ra,
                reading.dec), ljust=False)

    def _write_source_header(self):
        self._write_line("##   X        Y        X_0     Y_0          R.A.          DEC")

    def write_headers(self, observations, sys_header):
        """
        Writes the header part of the astrom file so that only the source
        data has to be filled in.
        """
        if self._header_written:
            raise AstromFormatError("Astrom file already has headers.")

        self._write_observation_list(observations)
        self._write_observation_headers(observations)
        self._write_sys_header(sys_header)
        self._write_source_header()

        self._header_written = True

    def flush(self):
        self.output_file.flush()

    def close(self):
        self.output_file.close()


class StreamingAstromWriter(BaseAstromWriter):
    """
    Use if you want to write out sources one-by-one as they are validated.
    See also BulkAstromWriter.
    """

    def __init__(self, filehandle, sys_header):
        super(StreamingAstromWriter, self).__init__(filehandle)
        self.sys_header = sys_header

        # Allow that the headers might have been written out in a previous
        # session by a different writer object.  In this case we just want
        # to be able to add more sources.
        self.output_file.seek(0)
        if re.match(OBS_LIST_PATTERN, self.output_file.read()):
            self._header_written = True

    def write_source(self, source):
        """
        Writes out data for a single source.
        """
        if not self._header_written:
            observations = [reading.get_observation() for reading in source.get_readings()]
            self.write_headers(observations, self.sys_header)

        self._write_source(source)


class BulkAstromWriter(BaseAstromWriter):
    """
    Use if you want to write out an entire AstromData structure at once.
    See also StreamingAstromWriter.
    """

    def __init__(self, filehandle):
        super(BulkAstromWriter, self).__init__(filehandle)

    def write_astrom_data(self, astrom_data):
        """
        Writes a full AstromData structure at once.
        """
        self.write_headers(astrom_data.observations, astrom_data.sys_header)
        self._write_source_data(astrom_data.sources)


class AstromData(object):
    """
    Encapsulates data extracted from an .astrom file.
    """

    def __init__(self, observations, sys_header, sources, discovery_only=False):
        """
        Constructs a new astronomy data set object.

        Args:
          observations: list(Observations)
            The observations that are part of the data set.
          sys_header: dict
            Key-value pairs of system settings applicable to the data set.
            Ex: RMIN, RMAX, ANGLE, AWIDTH
          sources: list(list(SourceReading))
            A list of point sources found in the data set.  These are
            potential moving objects.  Each point source is itself a list
            of source readings, one for each observation in
            <code>observations</code>.  By convention the ordering of
            source readings must match the ordering of the observations.
          discovery_only: bool
            should we only use the discovery images on the first pass?
        """
        self.observations = observations
        self.mpc_observations = {}
        self.sys_header = sys_header
        self.sources = [Source(reading_list, discovery_only=discovery_only) for reading_list in sources]

    def get_reading_count(self):
        count = 0
        for source in self.sources:
            count += source.num_readings()

        return count

    def get_sources(self):
        return self.sources

    def get_source_count(self):
        return len(self.get_sources())


class Source(object):
    """
    A collection of source readings.
    """

    def __init__(self, readings, provisional_name=None, discovery_only=False):
        self.readings = readings
        self.provisional_name = provisional_name
        self.discovery_only = discovery_only
        if 4 > self.num_readings() > 1:
            self.set_min_cutout()

    def set_min_cutout(self):
        x = []
        y = []
        for reading in self.readings:
            x.append(reading.x0)
            y.append(reading.y0)
        dx = max(x) - min(x)
        dy = max(y) - min(y)
        for reading in self.readings:
            reading.dx = dx+20
            reading.dy = dy+20

    def get_reading(self, index):
        return self.readings[index]

    def get_readings(self):
        return self.readings

    def num_readings(self):
        return len(self.readings)

    def has_provisional_name(self):
        return self.provisional_name is not None

    def get_provisional_name(self):
        return self.provisional_name

    def set_provisional_name(self, provisional_name):
        self.provisional_name = provisional_name


class Ellipse(object):
    """
    An ellipse region for use in DS9.

    a = semi-major axis
    b = semi-minor axis
    pa = position angle in degrees (East is 0).

    """

    def __init__(self, a, b, pa):
        """
        :param a: semi-major axis
        :type a: Quantity
        :param b: semi-minor axis
        :type b: Quantity
        :param pa: Position Angle (East is 0)
        :type pa: Quantity
        """
        assert isinstance(a, Quantity)
        assert isinstance(b, Quantity)
        assert isinstance(pa, Quantity)
        self._a = a
        self._b = b
        self._pa = pa

    def __str__(self):
        return '{}", {}", {}'.format(self.a.to(units.arcsec).value,
                                     self.b.to(units.arcsec).value,
                                     self.pa.to(units.degree).value + 90)

    @property
    def a(self):
        """
        The semi-major axis of the ellipse
        @return: Quantity
        """
        return self._a

    @a.setter
    def a(self, value):
        self._a = value

    @property
    def b(self):
        """
        The semi-minor axis of the ellipse.
        @return: Quantity
        """
        return self._b

    @b.setter
    def b(self, value):
        self._b = value

    @property
    def pa(self):
        """
        The orientation angle of the ellipse, counter-clock wise from due west.
        @return: Quantity
        """
        return self._pa

    @pa.setter
    def pa(self, value):
        self._pa = value


class SourceReading(object):
    """
    Data for a detected point source (which is a potential moving objects).
    """

    def __init__(self, x, y, x0, y0, ra, dec, xref, yref, obs, ssos=False, from_input_file=False,
                 null_observation=False, discovery=False, dx=0, dy=0, pa=0):
        """
        :param pa:
        :rtype : SourceReading
        :param obs: the Observation object associated with this measurement
        :type obs: Observation
        Args:
          x, y: the coordinates of the source in this reading.
          x0, y0: the coordinates of the source in this reading, but in
            the coordinate frame of the reference image.
          ra: right ascension
          dec: declination
          xref, yref: coordinates of the source in the reference image, in
            the reference image's coordinate frame.
          obs: the observation in which this reading was taken.
          naxis1, naxis2: the size of the FITS image where this detection is from.
        @param is_inverted:
        """
        # print x, y, x0, y0, ra, dec, xref, yref
        self._pix_coord = None
        if x is not None and y is not None:
            self.pix_coord = x, y
        self._ref_coord = None
        if x0 is not None and y0 is not None:
            self.ref_coord = x0, y0
        self._sky_coord = None
        self.sky_coord = ra, dec
        self.xref = xref
        self.yref = yref
        self._uncertainty_ellipse = None
        self._inverted = None
        self.uncertainty_ellipse = dx, dy, pa
        self._obs = None
        self.obs = obs
        self._ssos = None
        self.ssos = ssos
        self._from_input_file = None
        self.from_input_file = from_input_file
        self.null_observation = null_observation
        self._discovery = None
        self.discovery = discovery
        self.mpc_observation = None
        self.mpc_observations = {}
        self.reference_sky_coord = self.sky_coord
        self.min_cutout = 0.3 * units.arcminute

    def _original_frame(self, x, y):
        """
        Return x/y in the original frame, based on a guess as much as anything.
        :param x: x pixel coordinate
        :type x: float
        :param y: y pixel coordinate
        :type y: float
        :return: x,y
        :rtype: float, float
        """
        if self._inverted:
            return self.obs.naxis1 - x, self.obs.naxis2 - y
        return x, y

    @property
    def obs(self):
        """
        :return: The Observation that provide the current source reading.
        :rtype: Observation
        """
        return self._obs

    @obs.setter
    def obs(self, obs):
        """
        :param obs: The observation the provided this source reading.
        :type obs: Observation
        :return:
        """
        assert isinstance(obs, Observation)
        self._obs = obs

    @property
    def x_ref_offset(self):
        return self.x - self.x0

    @property
    def y_ref_offset(self):
        return self.y - self.y0

    @property
    def pix_coord(self):
        """

        :return: The x,y pixel location of the source in the current frame.
        :rtype: (Quantity, Quantity)
        """
        return self._pix_coord

    @pix_coord.setter
    def pix_coord(self, pix_coord):
        """
        :type pix_coord: list
        :param pix_coord: an x,y pixel coordinate, origin = 1
        """
        try:
            pix_coord = list(pix_coord)
        except:
            pass
        if not isinstance(pix_coord, list) or len(pix_coord) != 2:
            raise ValueError("pix_coord needs to be set with an (x,y) coordinate pair, got {}".format(pix_coord))
        x, y = pix_coord
        if not isinstance(x, Quantity):
            x = float(x) * units.pix
        if not isinstance(y, Quantity):
            y = float(y) * units.pix
        self._pix_coord = x, y

    @property
    def x(self):
        """
        :return: the x coordinate value
        :rtype: float
        """
        return self.pix_coord[0].value

    @property
    def y(self):
        """
        :return: the y coordinate value
        :rtype: float
        """
        return self.pix_coord[1].value

    @property
    def ref_coord(self):
        """

        :return: The x,y pixel location of the source in the reference frame.
        :rtype: (Quantity, Quantity)
        """
        return self._ref_coord

    @ref_coord.setter
    def ref_coord(self, pix_coord):
        """
        :type pix_coord: list
        :param pix_coord: an x,y pixel coordinate, origin = 1
        """
        try:
            pix_coord = list(pix_coord)
        except:
            pass
        if not isinstance(pix_coord, list) or len(pix_coord) != 2:
            raise ValueError("pix_coord needs to be set with an (x,y) coordinate pair, got {}".format(pix_coord))
        x, y = pix_coord
        if not isinstance(x, Quantity):
            x = float(x) * units.pix
        if not isinstance(y, Quantity):
            y = float(y) * units.pix
        self._ref_coord = x, y

    @property
    def x0(self):
        return self._ref_coord[0].value

    @property
    def y0(self):
        return self._ref_coord[1].value

    @property
    def sky_coord(self):
        """

        :return: the world coordinate longitude location.
        :rtype: SkyCoord
        """
        return self._sky_coord

    @property
    def ra(self):
        return self.sky_coord.ra.degree

    @property
    def dec(self):
        return self.sky_coord.dec.degree

    @sky_coord.setter
    def sky_coord(self, sky_coord):
        try:
            sky_coord = list(sky_coord)
        except:
            pass
        if isinstance(sky_coord, list):
            ra, dec = sky_coord
            if not isinstance(ra, Quantity):
                ra = float(ra)*units.degree
                dec = float(dec)*units.degree
            sky_coord = SkyCoord(ra, dec, 1)
        if not isinstance(sky_coord, SkyCoord):
            raise ValueError("Failed to initialize coordinate using {}".format(sky_coord))
        self._sky_coord = sky_coord

    @property
    def uncertainty_ellipse(self):
        """

        :return: The semi-major axis, semi-minor axis and position angle of the uncertainty ellipse
        :rtype: Ellipse
        """
        return self._uncertainty_ellipse

    @uncertainty_ellipse.setter
    def uncertainty_ellipse(self, ellipse):
        try:
            ellipse = list(ellipse)
        except:
            pass
        if not isinstance(ellipse, list) or len(ellipse) != 3:
            raise ValueError("Don't know how to set ellipse using: {}".format(ellipse))
        a, b, pa = ellipse
        if not isinstance(a, Quantity):
            a = float(a) * units.arcsecond
        if not isinstance(b, Quantity):
            b = float(b) * units.arcsecond
        if not isinstance(pa, Quantity):
            pa = float(pa) * units.degree
        self._uncertainty_ellipse = Ellipse(a, b, pa)

    @property
    def from_input_file(self):
        return self._from_input_file

    @from_input_file.setter
    def from_input_file(self, from_input_file):
        self._from_input_file = from_input_file

    def __repr__(self):
        return "<SourceReading x=%s, y=%s, x0=%s, y0=%s, ra=%s, dec=%s, obs=%s" % (
            self.x, self.y, self.x0, self.y0, self.ra, self.dec, self.obs)

    @property
    def source_point(self):
        return self.x, self.y

    @property
    def reference_source_point(self):
        """
        The location of the source in the reference image, in terms of the
        current image coordinates.
        """
        xref = isinstance(self.xref, Quantity) and self.xref.value or self.xref
        yref = isinstance(self.yref, Quantity) and self.yref.value or self.yref

        return xref + self.x_ref_offset, yref + self.y_ref_offset

    def get_observation_header(self):
        return self.obs.header

    @staticmethod
    def get_original_image_size():
        raise NotImplemented

    def get_exposure_number(self):
        return self.obs.expnum

    def get_observation(self):
        return self.obs

    def get_coordinate_offset(self, other_reading):
        """
        Calculates the offsets between readings' coordinate systems.

        Args:
          other_reading: ossos.astrom.SourceReading
            The reading to compare coordinate systems with.

        Returns:
          (offset_x, offset_y):
            The x and y offsets between this reading and the other reading's
            coordinate systems.
        """
        my_x, my_y = self.reference_source_point
        other_x, other_y = other_reading.reference_source_point
        return my_x - other_x, my_y - other_y

    def get_image_uri(self):
        return self.obs.get_image_uri()

    def get_apcor_uri(self):
        return self.obs.get_apcor_uri()

    def get_zmag_uri(self):
        return self.obs.get_zmag_uri()

    def get_ccd_num(self):
        """
        Returns:
          ccdnum: int
            The number of the CCD that the image is on.
        """
        return int(self.obs.ccdnum)

    def get_extension(self):
        """
        Returns:
          extension: str
            The FITS file extension.
        """
        if self.obs.is_fake():
            # We get the image from the CCD directory and it is not
            # multi-extension.
            return 0

        # NOTE: ccd number is the extension, BUT Fits file extensions start at 1
        # Therefore ccd n = extension n + 1
        return str(self.get_ccd_num() + 1)

    @property
    def inverted(self):
        if  self._inverted is None:
            self.inverted = self.compute_inverted()
        return self._inverted

    @inverted.setter
    def inverted(self, inverted):
        self._inverted = inverted

    def compute_inverted(self):
        """
        Returns:
          inverted: bool
            True if the stored image is inverted.
        """
        # astheader = storage.get_astheader(self.obs.expnum, self.obs.ccdnum, version=self.obs.ftype)
        # pvwcs = wcs.WCS(astheader)
        # (x, y) = pvwcs.sky2xy(self.ra, self.dec)
        # logger.debug("is_inverted: X,Y {},{}  -> wcs X,Y {},{}".format(self.x, self.y, x, y))
        # dr2 = ((x-self.x)**2 + (y-self.y)**2)
        # return dr2 > 2

        if self.ssos or self.obs.is_fake() or self.obs.ftype == 's':
            inverted = False
        else:
            inverted = True if self.get_ccd_num() - 1 in INVERTED_CCDS else False
        logger.debug("Got that {} is_inverted: {}".format(self.obs.rawname, inverted))
        return inverted


class Observation(object):
    """
    Stores data for a single observation (which may be associated with many
    point sources/readings).  The observation refers the frame taken at the telescope.
    """

    # Aliases for useful header keys
    HEADER_IMG_SIZE_X = NAX1
    HEADER_IMG_SIZE_Y = NAX2

    @staticmethod
    def from_parse_data(rawname, fk, expnum, ftype, ccdnum):
        assert rawname == fk + expnum + ftype + ccdnum
        return Observation(expnum, ftype, ccdnum, fk)

    @staticmethod
    def from_source_reference(expnum, ccd, x, y):
        """
        Given the location of a source in the image, create an Observation.
        """

        image_uri = storage.dbimages_uri(expnum=expnum,
                                         ccd=None,
                                         version='p',
                                         ext='.fits',
                                         subdir=None)
        logger.debug('Trying to access {}'.format(image_uri))

        if not storage.exists(image_uri, force=False):
            logger.warning('Image not in dbimages? Trying subdir.')
            image_uri = storage.dbimages_uri(expnum=expnum,
                                             ccd=ccd,
                                             version='p')

        if not storage.exists(image_uri, force=False):
            logger.warning("Image doesn't exist in ccd subdir. %s" % image_uri)
            return None

        if x == -9999 or y == -9999:
            logger.warning("Skipping {} as x/y not resolved.".format(image_uri))
            return None

        mopheader_uri = storage.dbimages_uri(expnum=expnum,
                                             ccd=ccd,
                                             version='p',
                                             ext='.mopheader')
        if not storage.exists(mopheader_uri, force=False):
            # ELEVATE! we need to know to go off and reprocess/include this image.
            logger.critical('Image exists but processing incomplete. Mopheader missing. {}'.format(image_uri))
            return None

        # Build astrom.Observation
        observation = Observation(expnum=str(expnum),
                                  ftype='p',
                                  ccdnum=str(ccd),
                                  fk="")

        # JJK commented this out, I think the following line is not true?
        # observation.rawname = os.path.splitext(os.path.basename(image_uri))[0]+str(ccd).zfill(2)

        return observation

    def __init__(self, expnum, ftype, ccdnum, fk=None, image_uri=None):
        self.expnum = expnum
        self.fk = fk is not None and fk or ""
        self._header = None
        self.ccdnum = ccdnum is not None and str(ccdnum) or None
        self.ftype = ftype is not None and str(ftype) or None
        self.rawname = "{}{}{}{}".format(self.fk, self.expnum, ftype is not None and ftype or "",
                                         ccdnum is not None and str(ccdnum).zfill(2) or "")
        logger.debug(self.rawname)
        if image_uri is None:
            self.image_uri = self.get_image_uri()

    def __repr__(self):
        return "<Observation rawname=%s>" % self.rawname

    def is_fake(self):
        return self.fk == FAKE_PREFIX or self.ftype == 's'

    # TODO Remove get_image_uri from here, use the storage methods.
    def get_image_uri(self):
        if self.ftype == 'p' and (self.fk is None or self.fk == ''):
            return storage.dbimages_uri(self.expnum)

        return storage.dbimages_uri(self.expnum,
                                    ccd=self.ccdnum,
                                    version=self.ftype,
                                    prefix=self.fk,
                                    ext='.fits')

    def get_object_planted_uri(self):
        dir = os.path.dirname(storage.dbimages_uri(self.expnum, ccd=self.ccdnum))
        return dir+"/Object.planted"

    def get_apcor_uri(self):
        return storage.dbimages_uri(self.expnum,
                                    ccd=self.ccdnum,
                                    version=self.ftype,
                                    prefix=self.fk,
                                    ext=storage.APCOR_EXT)

    def get_zmag_uri(self):
        return storage.dbimages_uri(self.expnum,
                                    ccd=self.ccdnum,
                                    version=self.ftype,
                                    prefix=self.fk,
                                    ext=storage.ZEROPOINT_USED_EXT)

    @property
    def astheader(self):
        return storage.get_astheader(self.expnum, self.ccdnum, self.ftype, self.fk)

    @property
    def naxis1(self):
        return int(self.header[self.HEADER_IMG_SIZE_X])

    @property
    def naxis2(self):
        return int(self.header[self.HEADER_IMG_SIZE_Y])

    @property
    def header(self):
        if self._header is None:
            try:
                self._header = storage.get_mopheader(self.expnum, self.ccdnum, self.ftype, self.fk)
            except Exception as ex:
                logger.error(str(ex))
                self._header = self.astheader
        return self._header

    def get_mpc_date(self):
        header = self.header
        if isinstance(header, list):
            extno = self.ccdnum - 1
            print "Reading extension {} looking for header of CCD {}".format(extno, self.ccdnum)
            header = header[extno]
        mpc_date = header.get('MJD_OBS_CENTER', None)
        if mpc_date is None and 'MJD-OBS' in header:
            mjd_obs = float(header.get('MJD-OBS'))
            exptime = float(header.get('EXPTIME'))
            mpc_date = Time(mjd_obs,
                            format='mjd',
                            scale='utc',
                            precision=6)
            mpc_date += TimeDelta(exptime * units.second) / 2.0
            mpc_date = mpc_date.mpc
        return mpc_date


class VettingWriter(BaseAstromWriter):
    """
    Write out the 'vetting' format when looking for slow moving 'stationary' objects
    """

    def _write_headers(self, observations, sys_header):
        pass

    def _write_source(self, source):

        readings = source.get_readings()
        line = "{} {} {}".format(readings[0].object_id,
                                 readings[0].ra,
                                 readings[0].dec)
        for reading in source.get_readings():
            line += " {}".format(reading.obs.rawname)
        self._write_line(line)


class StreamingVettingWriter(VettingWriter):
    """
    Use if you want to write out sources one-by-one as they are validated.
    See also BulkAstromWriter.

    The Vetting writer doesn't have a header.
    """

    def __init__(self, filehandle, sys_header=None):
        super(StreamingVettingWriter, self).__init__(filehandle)
        self.sys_header = sys_header
        self._header_written = True

    def write_source(self, source):
        """
        Writes out data for a single source.
        """
        if not self._header_written:
            observations = [reading.get_observation() for reading in source.get_readings()]
            self.write_headers(observations, self.sys_header)

        self._write_source(source)




__author__ = "David Rusk <drusk@uvic.ca>"

import wx

from mock import Mock
from astropy.io import fits

from tests.manual_inspection import context
from ossos.fitsviewer.singletviewer import SingletViewer


TEST_FILE = "cutout_1200_2400_1350_2300-1616681p.fits"


def main():
    app = wx.App()
    rootframe = wx.Frame(None)

    viewer = SingletViewer(rootframe)

    hdulist = fits.open(context.get_test_data_path(TEST_FILE))
    fits_image = Mock()
    fits_image.as_hdulist.return_value = hdulist

    viewer.display(fits_image)

    viewer.draw_marker(50, 50, 10)
    viewer.draw_marker(20, 70, 5)

    rootframe.Show()
    app.MainLoop()


if __name__ == "__main__":
    main()



__author__ = "David Rusk <drusk@uvic.ca>"

import os
import sys

root_test_dir = os.path.dirname(os.path.dirname(__file__))
sys.path.insert(0, os.path.dirname(root_test_dir))

from tests import testutil


def get_test_data_path(filename):
    return os.path.join(root_test_dir, "../data", filename)




__author__ = "David Rusk <drusk@uvic.ca>"

import unittest

import wx
from mock import Mock
from hamcrest import assert_that, equal_to

from tests.base_tests import WxWidgetTestCase
from ossos.fitsviewer.singletviewer import SingletViewer
from ossos.gui.controllers import AbstractController
from ossos.gui.views.appview import ApplicationView


class ApplicationViewTest(WxWidgetTestCase):
    def setUp(self):
        super(ApplicationViewTest, self).setUp()

        class TestFactory(object):
            def create_controller(self, view):
                return Mock(spec=AbstractController)

        self.appview = ApplicationView(TestFactory())
        self.controller = self.appview.controller

    def tearDown(self):
        self.appview.close()

    def mock_image_viewer(self):
        image_viewer = Mock(spec=SingletViewer)
        self.appview.image_view_manager.image_viewer = image_viewer
        return image_viewer

    def test_press_exit(self):
        menubar = self.appview.mainframe.GetMenuBar()
        filemenu = menubar.GetMenu(menubar.FindMenu("File"))
        exit_button = self.get_menuitem_by_label(filemenu, "Exit")

        # Fire menu selection event
        event = wx.CommandEvent(wx.wxEVT_COMMAND_MENU_SELECTED,
                                exit_button.GetId())
        self.appview.mainframe.ProcessEvent(event)

        assert_that(self.controller.on_exit.call_count, equal_to(1))

    def test_reset_colormap(self):
        image_viewer = self.mock_image_viewer()

        self.appview.reset_colormap()
        image_viewer.reset_colormap.assert_called_once_with()

    def test_toggle_reticule(self):
        image_viewer = self.mock_image_viewer()

        self.appview.toggle_reticule()
        image_viewer.toggle_reticule.assert_called_once_with()


if __name__ == '__main__':
    unittest.main()



import itertools

__author__ = 'jjk'

from datetime import datetime
import time
import numpy

from astropy.time import erfa_time as sofa_time
from astropy.time import TimeString
from astropy.time import Time

class TimeMPC(TimeString):
    """
    Override the TimeString class to convert from MPC format string to astropy.time.Time object.

    usage:

    from astropy.time import Time
    Time.FORMATS[TimeMPC.name] = TimeMPC

    t = Time('2000 01 01.00001', format='mpc', scale='utc')

    str(t) == '2000 01 01.000001'
    """

    name = 'mpc'
    subfmts = (('mpc', '%Y %m %d', "{year:4d} {mon:02d} {day:02d}.{fracday:s}"),)

    ### need our own 'set_jds' function as the MPC Time string is not typical
    def set_jds(self, val1, val2):
        """

        Parse the time strings contained in val1 and set jd1, jd2

        :param val1: array of strings to parse into JD format
        :param val2: not used for string conversions but passed regardless
        """
        n_times = len(val1)  # val1,2 already checked to have same len
        iy = numpy.empty(n_times, dtype=numpy.intc)
        im = numpy.empty(n_times, dtype=numpy.intc)
        iday = numpy.empty(n_times, dtype=numpy.intc)
        ihr = numpy.empty(n_times, dtype=numpy.intc)
        imin = numpy.empty(n_times, dtype=numpy.intc)
        dsec = numpy.empty(n_times, dtype=numpy.double)

        # Select subformats based on current self.in_subfmt
        subfmts = self._select_subfmts(self.in_subfmt)

        for i, timestr in enumerate(val1):
            # Assume that anything following "." on the right side is a
            # floating fraction of a day.
            try:
                idot = timestr.rindex('.')
            except:
                fracday = 0.0
            else:
                timestr, fracday = timestr[:idot], timestr[idot:]
                fracday = float(fracday)

            for _, strptime_fmt, _ in subfmts:
                try:
                    tm = time.strptime(timestr, strptime_fmt)
                except ValueError:
                    pass
                else:
                    iy[i] = tm.tm_year
                    im[i] = tm.tm_mon
                    iday[i] = tm.tm_mday
                    ihr[i] = tm.tm_hour + int(24 * fracday)
                    imin[i] = tm.tm_min + int(60 * (24 * fracday - ihr[i]))
                    dsec[i] = tm.tm_sec + 60 * (60 * (24 * fracday - ihr[i]) - imin[i])
                    break
            else:
                raise ValueError('Time {0} does not match {1} format'
                .format(timestr, self.name))

        self.jd1, self.jd2 = sofa_time.dtf_jd(self.scale.upper().encode('utf8'),
                                              iy, im, iday, ihr, imin, dsec)
        return

    def str_kwargs(self):
        """                                                                                                                                           
        Generator that yields a dict of values corresponding to the                                                                                   
        calendar date and time for the internal JD values.

        Here we provide the additional 'fracday' element needed by 'mpc' format
        """
        iys, ims, ids, ihmsfs = sofa_time.jd_dtf(self.scale.upper()
                                                 .encode('utf8'),
                                                 6,
                                                 self.jd1, self.jd2)

        # Get the str_fmt element of the first allowed output subformat                                                                               
        _, _, str_fmt = self._select_subfmts(self.out_subfmt)[0]

        yday = None
        has_yday = '{yday:' in str_fmt or False

        for iy, im, iday, ihmsf in itertools.izip(iys, ims, ids, ihmsfs):
            ihr, imin, isec, ifracsec = ihmsf
            if has_yday:
                yday = datetime(iy, im, iday).timetuple().tm_yday

            # MPC uses day fraction as time part of datetime
            fracday = (((((ifracsec / 1000000.0 + isec) / 60.0 + imin) / 60.0) + ihr) / 24.0) * (10 ** 6)
            fracday = '{0:06g}'.format(fracday)[0:self.precision]
            #format_str = '{{0:g}}'.format(self.precision)
            #fracday = int(format_str.format(fracday))
            yield dict(year=int(iy), mon=int(im), day=int(iday), hour=int(ihr), min=int(imin), sec=int(isec),
                       fracsec=int(ifracsec), yday=yday, fracday=fracday)


Time.FORMATS[TimeMPC.name] = TimeMPC




import os
import sys
from ossos.__version__ import version

from setuptools import setup, find_packages

dependencies = ['pyraf >= 2.1.1',
                    'astropy >= 0.2.5',
                    'vos >= 2.0',
                    'ephem',
                    'requests >= 2.7',
                    'pyOpenSSL',
                    'numpy >= 1.6.1',
                    'wxPython-common > 2.8.1',
                    'matplotlib',
                    'Polygon2',
                    'd2to1 >= 0.2.10', 'scipy', 'uncertainties', 'pyds9 >= 1.8'
                        ]


if sys.version_info[0] > 2:
    print 'The MOP package is only compatible with Python version 2.7+, not yet with 3.x'
    sys.exit(-1)

# # Build the list of scripts to be installed.
script_dir = 'tools'
scripts = []
for script in os.listdir(script_dir):
    if script[-1] in ["~", "#"]:
        continue
    scripts.append(os.path.join(script_dir, script))


setup(name='ossos',
      version=version,
      url='http://github.com/OSSOS/MOP',
      author='''JJ Kavelaars (jjk@uvic.ca),
              Michele Bannister (micheleb@uvic.ca),
              David Rusk''',
      maintainer='M Bannister and JJ Kavelaars',
      maintainer_email='jjk@uvic.ca',
      description="Outer Solar System Origins Survey (OSSOS)",
      long_description='See http://www.ossos-survey.org/ for science details.',
      classifiers=['Intended Audience :: Science/Research',
                   'Topic :: Scientific/Engineering :: Astronomy',
                   'Development Status :: 4 - Beta',
                   'Programming Language :: Python :: 2 :: Only',
                   'Operating System :: MacOS :: MacOS X',
                   'Environment :: X11 Applications',
                   'License :: OSI Approved :: GNU General Public License (GPL)',
                   ],
      package_data={'ossos': ['gui/*.json']},
      dependency_links=['git+https://github.com/ericmandel/pyds9.git#egg=pyds9-1.8'],
      install_requires=dependencies,
      scripts=scripts,
      packages=find_packages(exclude=['tests',])
      )



from astropy import units
from xml.dom import minidom
import xml
from astropy.coordinates import SkyCoord

COLUMN_SEPARATOR = "|"


def create_astrores_document():
    implementation = xml.dom.getDOMImplementation()
    doctype = implementation.createDocumentType('ASTRO',
                                                None,
                                                "http://vizier.u-strasbg.fr/xml/astrores.dtd")
    dom = implementation.createDocument("http://vizier.u-strasbg.fr/doc/astrores.htx", "ASTRO", doctype)
    dom.getElementsByTagName("ASTRO")[0].setAttribute("ID", "v0.8")
    dom.getElementsByTagName("ASTRO")[0].setAttribute("xmlns:ASTRO", "http://vizier.u-strasbg.fr/doc/astrores.htx")
    assert isinstance(dom, minidom.Document)
    return dom


class EphemTarget(object):

    nodes = {"NAME": "Ephemeris",
             "TITLE": "Ephemeris for CFHT QSO"}
    fields = {"DATE_UTC": {"attr": {"datatype": "A", "width": "19", "format": "YYYY-MM-DD hh:mm:ss"},
                           "DESCRIPTION": "UTC Date"},
              "RA_J2000": {"attr": {"datatype": "A", "width": "11", "format": "RAh:RAm:RAs", "unit": "h"},
                           "DESCRIPTION": "Right ascension of target"},
              "DEC_J2000": {"attr": {"datatype": "A", "width": "11", "format": "DEd:DEm:DEs", "unit": "deg"},
                            "DESCRIPTION": "Declination of target"}}

    def __init__(self, name, column_separator=COLUMN_SEPARATOR, format='CFHT ET'):
        """
        create an ephmeris target, either with a 'orbfit' object or some mean rate of motion.

        :param name: a string containing the name of the target.
        """

        self.name = str(name)
        self.format = format
        self.doc = create_astrores_document()
        self.column_separator = column_separator
        self.coordinates = []

    def _init_cfht_et_file(self):
        self.doc = create_astrores_document()
        table = self.doc.createElement("TABLE")
        table.setAttribute("ID", "Table")
        self.doc.lastChild.appendChild(table)

        self.nodes = EphemTarget.nodes
        self.fields = EphemTarget.fields
        nodes = self.nodes
        fields = self.fields
        self.field_names = ["DATE_UTC", "RA_J2000", "DEC_J2000"]

        for (key, value) in nodes.iteritems():
            element = self.doc.createElement(key)
            element.appendChild(self.doc.createTextNode(value))
            table.appendChild(element)

        table.getElementsByTagName("TITLE")[0].lastChild.appendData(" target {}".format(self.name))

        table.appendChild(self.doc.createComment("Definition of each field"))

        for fieldName in self.field_names:
            field = self.doc.createElement("FIELD")
            field.setAttribute("name", fieldName)
            for (key, value) in fields[fieldName]['attr'].iteritems():
                field.setAttribute(key, value)
            description = self.doc.createElement("DESCRIPTION")
            description.appendChild(self.doc.createTextNode(fields[fieldName]['DESCRIPTION']))
            field.appendChild(description)
            table.appendChild(field)

        table.appendChild(self.doc.createComment("Data table"))

        data = self.doc.createElement("DATA")
        table.appendChild(data)

        header_lines = self._cdata_header(colsep=self.column_separator)

        csv = self.doc.createElement("CSV")
        csv.setAttribute("headlines", str(len(header_lines)))
        csv.setAttribute("colsep", self.column_separator)
        data.appendChild(csv)
        self.cdata = self.doc.createCDATASection("\n"+"\n".join(header_lines)+"\n")
        csv.appendChild(self.cdata)

    @staticmethod
    def _entry(value, width, colsep):
        return "{value:{width}.{width}}{colsep}".format(value=value, width=width, colsep=colsep)

    def _cdata_header(self, colsep="|"):
        """
        Create a header for the CDATA section, as a visual guide.
        """
        fields = self.fields
        header_lines = []
        line = ""
        for fieldName in self.field_names:
            width = int(fields[fieldName]['attr']['width'])
            line += self._entry(fieldName, width, colsep)
        header_lines.append(line)

        line = ""
        for fieldName in self.field_names:
            width = int(fields[fieldName]['attr']['width'])
            line += self._entry(fields[fieldName]['attr']['format'], width=width, colsep=colsep)
        header_lines.append(line)

        line = ""
        for fieldName in self.field_names:
            width = int(fields[fieldName]['attr']['width'])
            (l, m) = divmod(width, 10)
            guide = ""
            for i in range(l):
                guide += "".join(map(str, range(10)))
            guide += "".join(map(str, range(m)))
            line += self._entry(guide, width=width, colsep=colsep)
        header_lines.append(line)

        line = ""
        for fieldName in self.field_names:
            width = int(fields[fieldName]['attr']['width'])
            guide = "-" * width
            line += self._entry(guide, width=width, colsep=colsep)
        header_lines.append(line)

        return header_lines

    def append(self, coordinate):
        self.coordinates.append(coordinate)

    def _append_cdata(self, coordinate):
        """
        Append an target location to the ephemeris listing.
        """
        fields = self.fields
        sra = coordinate.ra.to_string(units.hour, sep=':', precision=2, pad=True)
        sdec = coordinate.dec.to_string(units.degree, sep=':', precision=1, alwayssign=True)
        coord = SkyCoord(sra+" "+sdec, unit=(units.hour, units.degree))
        sra = coord.ra.to_string(units.hour, sep=":", precision=2, pad=True)
        sdec = coord.dec.to_string(units.degree, sep=":", precision=1, pad=True, alwayssign=True)
        sdate = str(coordinate.obstime.replicate(format('iso')))
        self.cdata.appendData(self._entry(sdate, fields["DATE_UTC"]['attr']['width'], colsep=self.column_separator))
        self.cdata.appendData(self._entry(sra, fields["RA_J2000"]['attr']['width'], colsep=self.column_separator))
        self.cdata.appendData(self._entry(sdec, fields["DEC_J2000"]["attr"]["width"], colsep=self.column_separator))
        self.cdata.appendData("\n")

    def gemini_writer(self, f_handle):
        """
        Write out a GEMINI formated OT ephemeris.  This is just a hack of SSD Horizons output.
        """
        f_handle.write(GEMINI_HEADER)
        #Date__(UT)__HR:MN Date_________JDUT     R.A.___(ICRF/J2000.0)___DEC dRA*cosD d(DEC)/dt
        #         1         2         3         4         5         6         7         8         9
        #123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890
        #' 2019-Jan-30 00:00     01 46 56.46 +10 28 54.9 01 47 56.17 +10 34 27.6    3.520

        for coordinate in self.coordinates:
            date = coordinate.obstime.datetime.strftime('%Y-%b-%d %H:%M')[:17]
            f_handle.write(" {:16} {:17.9f}    {:27} {:+8.5f} {:+8.5f}\n".format(date,
                                                     coordinate.obstime.jd,
                                                     coordinate.to_string('hmsdms',
                                                                          sep=' ',
                                                                          precision=4,
                                                                          pad=True)[:27],
                                                     float(coordinate.dra),
                                                     float(coordinate.ddec)),
                           )
        f_handle.write(GEMINI_FOOTER)
        return

    def cfht_writer(self, f_handle):
        self._init_cfht_et_file()
        print len(self.coordinates)
        for coordinate in self.coordinates:
            self._append_cdata(coordinate)
        self.doc.writexml(f_handle, indent="  ", addindent="  ", newl='\n')

    def writer(self, f_handle):
            if self.format == 'CFHT ET':
                self.cfht_writer(f_handle)
            elif self.format == 'GEMINI ET':
                self.gemini_writer(f_handle)
            else:
                raise ValueError("unkown ET Format")

    def save(self, filename=None):
        if filename is None:
            if self.format == 'CFHT ET':
               filename = "ET_"+self.name+".xml"
            elif self.format == 'GEMINI ET':
               filename = self.name+".eph"
            else:
               filename = self.name+".txt"
        with file(filename, 'w') as f_handle:
            self.writer(f_handle)


GEMINI_HEADER="""*******************************************************************************
 Revised:  Jan 2, 2013                         Titan                       000
                         http://ssd.jpl.nasa.gov/?sat_phys_par
                           http://ssd.jpl.nasa.gov/?sat_elem
 SATELLITE PHYSICAL PROPERTIES:
  Mean Radius (km)       = 2575.5   +-  2.0  Density (g/cm^3) =  1.880 +- 0.004
  Mass (10^22 g)         = 13455.3           Geometric Albedo =  0.2 
  GM (km^3/s^2)          = 8978.13  +-  0.06  V(1,0)          = -1.2 
 SATELLITE ORBITAL DATA:
  Semi-major axis, a (km)= 1221.87 (10^3)  Orbital period     = 15.945421 d
  Eccentricity, e        = 0.0288          Rotational period  = 
  Inclination, i  (deg)  = 0.28
*******************************************************************************
 
 
*******************************************************************************
Ephemeris / WWW_USER Wed Oct 31 17:11:34 2012 Pasadena, USA      / Horizons    
*******************************************************************************
Target body name: Titan (606)                     {source: SAT351}
Center body name: Earth (399)                     {source: DE405}
Center-site name: Mauna Kea
*******************************************************************************
Start time      : A.D. 2012-Nov-01 00:00:00.0000 UT      
Stop  time      : A.D. 2013-Nov-01 00:00:00.0000 UT      
Step-size       : 60 minutes
*******************************************************************************
Target pole/equ : IAU_TITAN                       {East-longitude -}
Target radii    : 2575.0 x 2575.0 x 2575.0 km     {Equator, meridian, pole}    
Center geodetic : 204.527800,19.8261152,4.2078485 {E-lon(deg),Lat(deg),Alt(km)}
Center cylindric: 204.527800,6006.35451,2151.0229 {E-lon(deg),Dxy(km),Dz(km)}
Center pole/equ : High-precision EOP model        {East-longitude +}
Center radii    : 6378.1 x 6378.1 x 6356.8 km     {Equator, meridian, pole}    
Target primary  : Saturn                          {source: DE405}
Interfering body: MOON (Req= 1737.400) km         {source: DE405}
Deflecting body : Sun, EARTH                      {source: DE405}
Deflecting GMs  : 1.3271E+11, 3.9860E+05 km^3/s^2                              
Atmos refraction: NO (AIRLESS)
RA format       : HMS
Time format     : BOTH
EOP file        : eop.121031.p130122                                           
EOP coverage    : DATA-BASED 1962-JAN-20 TO 2012-OCT-31. PREDICTS-> 2013-JAN-21
Units conversion: 1 AU= 149597870.691 km, c= 299792.458 km/s, 1 day= 86400.0 s 
Table cut-offs 1: Elevation (-90.0deg=NO ),Airmass (< 2.000=YES),Daylight (YES)
Table cut-offs 2: Solar Elongation (  0.0,180.0=NO )                           
***************************************************************************************
 Date__(UT)__HR:MN Date_________JDUT     R.A.___(ICRF/J2000.0)___DEC dRA*cosD d(DEC)/dt
***************************************************************************************
$$SOE
"""
GEMINI_FOOTER="""$$EOE
*******************************************************************************
Column meaning:
 
TIME

  Prior to 1962, times are UT1. Dates thereafter are UTC. Any 'b' symbol in
the 1st-column denotes a B.C. date. First-column blank (" ") denotes an A.D.
date. Calendar dates prior to 1582-Oct-15 are in the Julian calendar system.
Later calendar dates are in the Gregorian system.

  The uniform Coordinate Time scale is used internally. Conversion between
CT and the selected non-uniform UT output scale has not been determined for
UTC times after the next July or January 1st.  The last known leap-second
is used over any future interval.

  NOTE: "n.a." in output means quantity "not available" at the print-time.
 
 R.A._(ICRF/J2000.0)_DEC =
   J2000.0 astrometric right ascension and declination of target center.
Corrected for light-time. Units: HMS (HH MM SS.ff) and DMS (DD MM SS.f)
 
 R.A._(a-apparent)__DEC. =
   Airless apparent right ascension and declination of the target center with
respect to the Earth true-equator and the meridian containing the Earth true
equinox of date.  Corrected for light-time, gravitational deflection of light,
stellar aberration, precession & nutation.
   Units: HMS (HH MM SS.ff) and DMS (DD MM SS.f)
 
 Ang-diam =
   The equatorial angular width of the target body full disk, if it were
fully visible to the observer.  Units: ARCSECONDS


 Computations by ...
     Solar System Dynamics Group, Horizons On-Line Ephemeris System
     4800 Oak Grove Drive, Jet Propulsion Laboratory
     Pasadena, CA  91109   USA
     Information: http://ssd.jpl.nasa.gov/
     Connect    : telnet://ssd.jpl.nasa.gov:6775  (via browser)
                  telnet ssd.jpl.nasa.gov 6775    (via command-line)
     Author     : Jon.Giorgini@jpl.nasa.gov

*******************************************************************************
"""



__author__ = "David Rusk <drusk@uvic.ca>"

import unittest

from hamcrest import assert_that, equal_to
from mock import patch

from tests.base_tests import FileReadingTestCase
from ossos.gui.app import ProcessRealsApplication


class ValidationApplicationTest(FileReadingTestCase):
    @patch("ossos.gui.views.dialogs.show_empty_workload_dialog")
    def test_empty_workload_triggers_dialog(self, mock_dialog):
        working_directory = self.get_abs_path("data/empty")
        output_directory = working_directory
        ProcessRealsApplication(working_directory, output_directory)

        assert_that(mock_dialog.call_count, equal_to(1))


if __name__ == '__main__':
    unittest.main()



__author__ = "David Rusk <drusk@uvic.ca>"

import wx
import wx.lib.inspection

from tests.manual_inspection.context import testutil
from ossos.gui.views.validation import AcceptSourceDialog, RejectSourceDialog


def main():
    app = wx.App()
    rootframe = wx.Frame(None)

    # TODO helper test frame/panel for these test dialogs
    # (use in dialogs.py as well)
    panel = wx.Panel(rootframe, wx.ID_ANY)

    def onclick_accept_src(event):
        controller = testutil.Dummy("Accept Source Controller")
        note1_choices = ["n1a", "n1b"]
        note2_choices = ["n2a", "n2b", "n2c"]
        AcceptSourceDialog(panel, controller, "provisional-name-1",
                           "2012 01 01",
                           27.213, 31.2123, 123.4, "A",
                           note1_choices=note1_choices,
                           note2_choices=note2_choices).Show()

    def onclick_reject_src(event):
        controller = testutil.Dummy("Reject Source Controller")
        RejectSourceDialog(panel, controller).Show()

    sizer = wx.BoxSizer(wx.HORIZONTAL)

    accept_src_button = wx.Button(panel, id=wx.ID_ANY, label="Accept source dialog")
    accept_src_button.Bind(wx.EVT_BUTTON, onclick_accept_src)

    reject_src_button = wx.Button(panel, id=wx.ID_ANY, label="Reject source dialog")
    reject_src_button.Bind(wx.EVT_BUTTON, onclick_reject_src)

    sizer.Add(accept_src_button)
    sizer.Add(reject_src_button)

    panel.SetSizer(sizer)

    wx.lib.inspection.InspectionTool().Show()
    rootframe.Show()
    app.MainLoop()


if __name__ == "__main__":
    main()



__author__ = "David Rusk <drusk@uvic.ca>"

import os
import unittest

from astropy.io import fits
from hamcrest import (assert_that, equal_to, has_length, contains,
                      same_instance, is_not, contains_inanyorder)
from mock import patch, Mock

from tests.base_tests import FileReadingTestCase, DirectoryCleaningTestCase
from ossos.downloads.async import AsynchronousDownloadManager
from ossos.downloads.core import ApcorData
from ossos.downloads.cutouts.calculator import CoordinateConverter
from ossos.downloads.cutouts.source import SourceCutout
from ossos.gui.context import LocalDirectoryWorkingContext
from ossos.gui import events, tasks
from ossos.gui.models.exceptions import ImageNotLoadedException
from ossos.gui.models.imagemanager import ImageManager
from ossos.gui.models.validation import ValidationModel
from ossos.astrom import AstromParser
from ossos.gui.progress import LocalProgressManager
from ossos.gui.models.workload import (WorkUnitProvider, RealsWorkUnitBuilder,
                                       CandidatesWorkUnitBuilder)


MODEL_TEST_DIR_1 = "data/model_testdir_1"
MODEL_TEST_DIR_2 = "data/model_testdir_2"
MODEL_TEST_DIR_3 = "data/model_testdir_3"
EMPTY_DIR = "data/empty"

FRESH_TEST_DIR = "data/model_persistence_fresh"
TEST_FILES = ["xxx1.cands.astrom", "xxx2.cands.astrom", "xxx3.reals.astrom", "xxx4.reals.astrom"]

TEST_OBJECT_NAME = "O13AE0Z"


class GeneralModelTest(FileReadingTestCase, DirectoryCleaningTestCase):
    def setUp(self):
        events.unsub_all()

        parser = AstromParser()
        directory_manager = LocalDirectoryWorkingContext(self._get_working_dir())
        self.context = directory_manager
        progress_manager = LocalProgressManager(directory_manager)
        workunit_provider = WorkUnitProvider(tasks.get_suffix(self._get_task()),
                                             directory_manager, progress_manager,
                                             self._get_workunit_builder(parser, progress_manager))

        self.workunit_provider = workunit_provider
        self.progress_manager = progress_manager

        self.singlet_download_manager = Mock(spec=AsynchronousDownloadManager)
        self.triplet_download_manager = Mock(spec=AsynchronousDownloadManager)
        self.image_manager = ImageManager(self.singlet_download_manager,
                                          self.triplet_download_manager)

        self.model = ValidationModel(self.workunit_provider, self.image_manager, None)

        self.custom_setup()

        self.model.start_work()

    def custom_setup(self):
        pass

    def _get_task(self):
        raise NotImplementedError()

    def _get_working_dir(self):
        raise NotImplementedError()

    def get_directory_to_clean(self):
        return self._get_working_dir()

    def _get_workunit_builder(self, parser, progress_manager):
        raise NotImplementedError()

    def create_real_first_image(self, path="data/testimg.fits"):
        # Put a real fits image on the first source, first observation
        apcor = ApcorData.from_string("4 15   0.19   0.01")
        hdulist = fits.open(self.get_abs_path(path))
        first_reading = self.model.get_current_workunit().get_sources()[0].get_readings()[0]
        self.first_snapshot = SourceCutout(first_reading, hdulist, apcor)
        self.image_manager.on_singlet_image_loaded(self.first_snapshot)


class AbstractRealsModelTest(GeneralModelTest):
    def _get_working_dir(self):
        return self.get_abs_path(MODEL_TEST_DIR_1)

    def _get_task(self):
        return tasks.REALS_TASK

    def _get_workunit_builder(self, parser, progress_manager):
        return RealsWorkUnitBuilder(parser, self.context, self.context,
                                    progress_manager)

    def get_files_to_keep(self):
        return ["1584431p15.measure3.reals.astrom"]

    def test_sources_initialized(self):
        assert_that(self.model.get_current_source_number(), equal_to(0))
        assert_that(self.model.get_obs_count(), equal_to(3))

    def test_next_source_previous_source(self):
        assert_that(self.model.get_current_source_number(), equal_to(0))
        self.model.next_source()
        assert_that(self.model.get_current_source_number(), equal_to(1))
        self.model.previous_source()
        assert_that(self.model.get_current_source_number(), equal_to(0))

    def test_next_source_wrap(self):
        assert_that(self.model.get_current_source_number(), equal_to(0))
        self.model.next_source()
        assert_that(self.model.get_current_source_number(), equal_to(1))
        self.model.next_source()
        assert_that(self.model.get_current_source_number(), equal_to(2))
        self.model.next_source()
        assert_that(self.model.get_current_source_number(), equal_to(0))

    def test_next_obs(self):
        assert_that(self.model.get_current_obs_number(), equal_to(0))
        self.model.next_obs()
        assert_that(self.model.get_current_obs_number(), equal_to(1))
        self.model.next_obs()
        assert_that(self.model.get_current_obs_number(), equal_to(2))
        self.model.next_obs()
        assert_that(self.model.get_current_obs_number(), equal_to(0))

    def test_previous_source_wrap(self):
        assert_that(self.model.get_current_source_number(), equal_to(0))
        self.model.previous_source()
        assert_that(self.model.get_current_source_number(), equal_to(2))
        self.model.previous_source()
        assert_that(self.model.get_current_source_number(), equal_to(1))
        self.model.previous_source()
        assert_that(self.model.get_current_source_number(), equal_to(0))

    def test_previous_obs(self):
        assert_that(self.model.get_current_obs_number(), equal_to(0))
        self.model.previous_obs()
        assert_that(self.model.get_current_obs_number(), equal_to(2))
        self.model.previous_obs()
        assert_that(self.model.get_current_obs_number(), equal_to(1))
        self.model.previous_obs()
        assert_that(self.model.get_current_obs_number(), equal_to(0))

    def test_next_source_resets_obs(self):
        assert_that(self.model.get_current_obs_number(), equal_to(0))
        self.model.next_obs()
        assert_that(self.model.get_current_obs_number(), equal_to(1))
        self.model.next_source()
        assert_that(self.model.get_current_obs_number(), equal_to(0))
        self.model.next_obs()
        assert_that(self.model.get_current_obs_number(), equal_to(1))

    def test_receive_next_source_event(self):
        # Subscribe a mock
        observer = Mock()
        events.subscribe(events.CHANGE_IMAGE, observer.on_next_event)

        # Perform action
        self.model.next_source()

        # Make sure event triggered
        observer.on_next_event.assert_called_once()

    def test_receive_next_obs_event(self):
        # Subscribe a mock
        observer = Mock()
        events.subscribe(events.CHANGE_IMAGE, observer.on_change_img)

        # Perform action
        self.model.next_obs()

        # Make sure event triggered
        observer.on_change_img.assert_called_once()

    def test_receive_previous_source_event(self):
        # Subscribe a mock
        observer = Mock()
        events.subscribe(events.CHANGE_IMAGE, observer.on_change_img)

        # Perform actions
        self.model.next_source()
        self.model.previous_source()

        # Make sure event triggered
        observer.on_change_img.assert_called_once()

    def test_receive_previous_source_event(self):
        # Subscribe a mock
        observer = Mock()
        events.subscribe(events.CHANGE_IMAGE, observer.on_change_img)

        # Perform actions
        self.model.next_obs()
        self.model.previous_obs()

        # Make sure event triggered
        observer.on_change_img.assert_called_once()

    def test_receive_nav_event_next_and_prev_source(self):
        # Subscribe a mock
        observer = Mock()
        events.subscribe(events.CHANGE_IMAGE, observer.on_nav)

        # Perform actions
        self.model.next_obs()
        self.model.previous_obs()

        # Make sure event triggered
        assert_that(observer.on_nav.call_count, equal_to(2))

    def test_get_reading_data(self):
        assert_that(self.model.get_reading_data(), contains(
            ("X", 911.00), ("Y", 3967.12), ("X_0", 911.00), ("Y_0", 3967.12),
            ("R.A.", 26.6833367), ("DEC", 29.2203532)
        ))

    def test_loading_images(self):
        observer = Mock()
        events.subscribe(events.IMG_LOADED, observer.on_img_loaded)
        image1 = Mock()
        loaded_reading1 = image1.reading
        image2 = Mock()
        loaded_reading2 = image2.reading

        assert_that(
            self.image_manager._singlet_download_manager.submit_request.call_count,
            equal_to(9))

        # Simulate receiving callback
        self.image_manager.on_singlet_image_loaded(image1)
        assert_that(observer.on_img_loaded.call_count, equal_to(1))

        # Simulate receiving callback
        self.image_manager.on_singlet_image_loaded(image2)
        assert_that(observer.on_img_loaded.call_count, equal_to(2))

        # Check event args
        call_args_list = observer.on_img_loaded.call_args_list
        assert_that(call_args_list, has_length(2))

        msg0 = call_args_list[0][0][0]
        assert_that(msg0.topic, equal_to(events.IMG_LOADED))
        assert_that(msg0.data, equal_to(loaded_reading1))

        msg1 = call_args_list[1][0][0]
        assert_that(msg1.topic, equal_to(events.IMG_LOADED))
        assert_that(msg1.data, equal_to(loaded_reading2))

    def test_get_current_exposure_number(self):
        assert_that(self.model.get_current_exposure_number(), equal_to(1584431))
        self.model.next_obs()
        assert_that(self.model.get_current_exposure_number(), equal_to(1584449))
        self.model.next_obs()
        assert_that(self.model.get_current_exposure_number(), equal_to(1584453))
        self.model.next_obs()
        assert_that(self.model.get_current_exposure_number(), equal_to(1584431))
        self.model.previous_obs()
        assert_that(self.model.get_current_exposure_number(), equal_to(1584453))

    def test_get_current_data(self):
        assert_that(self.model.get_current_observation_date(), equal_to("2012 10 21.40516"))
        assert_that(self.model.get_current_ra(), equal_to(26.6833367))
        assert_that(self.model.get_current_dec(), equal_to(29.2203532))

        self.model.next_obs()

        assert_that(self.model.get_current_observation_date(), equal_to("2012 10 21.48212"))
        assert_that(self.model.get_current_ra(), equal_to(26.6816808))
        assert_that(self.model.get_current_dec(), equal_to(29.2202748))

        self.model.next_source()
        self.model.next_obs()

        assert_that(self.model.get_current_observation_date(), equal_to("2012 10 21.48212"))
        assert_that(self.model.get_current_ra(), equal_to(26.6816808))
        assert_that(self.model.get_current_dec(), equal_to(29.2202748))

    def test_sources_processed(self):
        workunit = self.model.get_current_workunit()

        assert_that(self.model.get_num_items_processed(), equal_to(0))
        assert_that(self.model.get_current_source_number(), equal_to(0))

        self.model.accept_current_item()

        assert_that(self.model.get_current_source_number(), equal_to(0))
        assert_that(self.model.get_num_items_processed(), equal_to(1))

        source1 = self.model.get_current_data().get_sources()[0]
        assert_that(workunit.is_item_processed(source1.get_reading(0)), equal_to(True))
        assert_that(workunit.is_item_processed(source1.get_reading(1)), equal_to(False))
        assert_that(workunit.is_item_processed(source1.get_reading(2)), equal_to(False))

        self.model.next_item()
        self.model.reject_current_item()

        assert_that(self.model.get_current_source_number(), equal_to(0))
        assert_that(self.model.get_num_items_processed(), equal_to(2))

    def test_get_current_band(self):
        self.create_real_first_image("data/1616681p22.fits")
        assert_that(self.model.get_current_band(), equal_to("r"))

    def test_get_current_image(self):
        self.create_real_first_image()
        assert_that(self.model.get_current_cutout(),
                    same_instance(self.first_snapshot))

    def test_get_current_snapshot_not_loaded(self):
        self.model.next_source()
        self.assertRaises(ImageNotLoadedException, self.model.get_current_cutout)

    def test_get_current_reading_data(self):
        self.model.next_source()
        self.model.next_obs()

        assert_that(self.model.get_current_ra(), equal_to(26.6816808))
        assert_that(self.model.get_current_dec(), equal_to(29.2202748))
        assert_that(self.model.get_current_image_fwhm(), equal_to(3.30))
        assert_that(self.model.get_current_image_maxcount(), equal_to(30000.0))


class ProcessRealsModelTest(GeneralModelTest):
    def _get_task(self):
        return tasks.REALS_TASK

    def _get_workunit_builder(self, parser, progress_manager):
        return RealsWorkUnitBuilder(parser, self.context, self.context,
                                    progress_manager)

    def get_files_to_keep(self):
        return ["1584431p15.measure3.reals.astrom"]

    def _get_working_dir(self):
        return self.get_abs_path(MODEL_TEST_DIR_1)

    def test_next_item_no_validation(self):
        observer = Mock()
        events.subscribe(events.CHANGE_IMAGE, observer.on_change_img)

        assert_that(self.model.get_current_source_number(), equal_to(0))
        assert_that(self.model.get_current_obs_number(), equal_to(0))

        self.model.next_item()
        assert_that(self.model.get_current_source_number(), equal_to(0))
        assert_that(self.model.get_current_obs_number(), equal_to(1))
        assert_that(observer.on_change_img.call_count, equal_to(1))

        self.model.next_item()
        assert_that(self.model.get_current_source_number(), equal_to(0))
        assert_that(self.model.get_current_obs_number(), equal_to(2))
        assert_that(observer.on_change_img.call_count, equal_to(2))

        self.model.next_item()
        # Should have looped back to first observation of the same source
        # because we haven't finished processing it.
        assert_that(self.model.get_current_source_number(), equal_to(0))
        assert_that(self.model.get_current_obs_number(), equal_to(0))
        assert_that(observer.on_change_img.call_count, equal_to(3))

    def test_next_item_after_validate_last(self):
        self.model.set_current_source_name(TEST_OBJECT_NAME)

        assert_that(self.model.get_current_source_number(), equal_to(0))
        assert_that(self.model.get_current_obs_number(), equal_to(0))

        self.model.next_item()
        assert_that(self.model.get_current_source_number(), equal_to(0))
        assert_that(self.model.get_current_obs_number(), equal_to(1))

        self.model.next_item()
        assert_that(self.model.get_current_source_number(), equal_to(0))
        assert_that(self.model.get_current_obs_number(), equal_to(2))

        self.model.accept_current_item()
        self.model.next_item()

        # Should have looped back to first observation of the same source
        # because we haven't finished processing it.
        assert_that(self.model.get_current_source_number(), equal_to(0))
        assert_that(self.model.get_current_obs_number(), equal_to(0))

        self.model.accept_current_item()
        self.model.next_item()

        assert_that(self.model.get_current_source_number(), equal_to(0))
        assert_that(self.model.get_current_obs_number(), equal_to(1))

        self.model.accept_current_item()
        self.model.next_item()

        # We already validated the last reading, so we should be jumping
        # straight to the second source now.

        assert_that(self.model.get_current_source_number(), equal_to(1))
        assert_that(self.model.get_current_obs_number(), equal_to(0))

        self.model.next_item()

        assert_that(self.model.get_current_source_number(), equal_to(1))
        assert_that(self.model.get_current_obs_number(), equal_to(1))

    def test_next_item_jump_over_processed(self):
        assert_that(self.model.get_current_source_number(), equal_to(0))
        assert_that(self.model.get_current_obs_number(), equal_to(0))

        self.model.next_item()
        assert_that(self.model.get_current_source_number(), equal_to(0))
        assert_that(self.model.get_current_obs_number(), equal_to(1))

        self.model.reject_current_item()
        self.model.next_item()

        assert_that(self.model.get_current_source_number(), equal_to(0))
        assert_that(self.model.get_current_obs_number(), equal_to(2))

        self.model.next_item()
        assert_that(self.model.get_current_source_number(), equal_to(0))
        assert_that(self.model.get_current_obs_number(), equal_to(0))

        self.model.next_item()
        assert_that(self.model.get_current_source_number(), equal_to(0))
        assert_that(self.model.get_current_obs_number(), equal_to(2))

    def test_is_source_discovered(self):
        self.model.set_current_source_name(TEST_OBJECT_NAME)

        assert_that(self.model.is_current_source_discovered(), equal_to(False))
        self.model.reject_current_item()
        assert_that(self.model.is_current_source_discovered(), equal_to(False))
        self.model.next_item()
        assert_that(self.model.is_current_source_discovered(), equal_to(False))
        self.model.accept_current_item()
        assert_that(self.model.is_current_source_discovered(), equal_to(True))
        self.model.next_item()
        assert_that(self.model.is_current_source_discovered(), equal_to(True))
        self.model.accept_current_item()
        assert_that(self.model.is_current_source_discovered(), equal_to(True))
        self.model.next_source()
        assert_that(self.model.is_current_source_discovered(), equal_to(False))

    def test_accept_current_item(self):
        workunit = self.model.get_current_workunit()

        data = self.model.get_current_data()
        first_item = data.get_sources()[0].get_readings()[0]
        second_item = data.get_sources()[0].get_readings()[1]

        assert_that(workunit.is_item_processed(first_item), equal_to(False))
        assert_that(workunit.is_item_processed(second_item), equal_to(False))

        self.model.accept_current_item()

        assert_that(workunit.is_item_processed(first_item), equal_to(True))
        assert_that(workunit.is_item_processed(second_item), equal_to(False))

        self.model.next_item()
        self.model.accept_current_item()

        assert_that(workunit.is_item_processed(first_item), equal_to(True))
        assert_that(workunit.is_item_processed(second_item), equal_to(True))

    def test_reject_current_item(self):
        workunit = self.model.get_current_workunit()

        data = self.model.get_current_data()
        first_item = data.get_sources()[0].get_readings()[0]
        second_item = data.get_sources()[0].get_readings()[1]

        assert_that(workunit.is_item_processed(first_item), equal_to(False))
        assert_that(workunit.is_item_processed(second_item), equal_to(False))

        self.model.reject_current_item()

        assert_that(workunit.is_item_processed(first_item), equal_to(True))
        assert_that(workunit.is_item_processed(second_item), equal_to(False))

        self.model.next_item()
        self.model.reject_current_item()

        assert_that(workunit.is_item_processed(first_item), equal_to(True))
        assert_that(workunit.is_item_processed(second_item), equal_to(True))

    def test_receive_all_sources_processed_event_on_final_accept(self):
        observer = Mock()
        events.subscribe(events.NO_AVAILABLE_WORK, observer.on_all_processed)

        total_items = 9
        item = 0
        while item < total_items - 1:
            self.model.set_current_source_name(TEST_OBJECT_NAME)
            self.model.accept_current_item()
            assert_that(observer.on_all_processed.call_count, equal_to(0))
            self.model.next_item()
            item += 1

        self.model.accept_current_item()
        # We try to get the next work unit, and find there is none
        self.model.next_item()

        assert_that(observer.on_all_processed.call_count, equal_to(1))

    def test_receive_all_sources_processed_event_on_final_reject(self):
        observer = Mock()
        events.subscribe(events.NO_AVAILABLE_WORK, observer.on_all_processed)

        total_items = 9
        item = 0
        while item < total_items - 1:
            self.model.set_current_source_name(TEST_OBJECT_NAME)
            self.model.accept_current_item()
            assert_that(observer.on_all_processed.call_count, equal_to(0))
            self.model.next_item()
            item += 1

        self.model.reject_current_item()

        # We try to get the next work unit, and find there is none
        self.model.next_item()
        assert_that(observer.on_all_processed.call_count, equal_to(1))


class ProcessCandidatesModelTest(GeneralModelTest):
    def _get_working_dir(self):
        return self.get_abs_path(MODEL_TEST_DIR_3)

    def _get_task(self):
        return tasks.CANDS_TASK

    def _get_workunit_builder(self, parser, progress_manager):
        return CandidatesWorkUnitBuilder(parser, self.context, self.context,
                                         progress_manager)

    def get_files_to_keep(self):
        return ["1584431p15.measure3.cands.astrom", "1584431p15.measure3.reals.astrom"]

    def test_next_item(self):
        observer = Mock()
        events.subscribe(events.CHANGE_IMAGE, observer.on_change_img)

        assert_that(self.model.get_current_source_number(), equal_to(0))
        assert_that(self.model.get_current_obs_number(), equal_to(0))

        self.model.next_item()
        assert_that(self.model.get_current_source_number(), equal_to(1))
        assert_that(self.model.get_current_obs_number(), equal_to(0))
        assert_that(observer.on_change_img.call_count, equal_to(1))

        self.model.next_item()
        assert_that(self.model.get_current_source_number(), equal_to(2))
        assert_that(self.model.get_current_obs_number(), equal_to(0))
        assert_that(observer.on_change_img.call_count, equal_to(2))

        self.model.next_item()
        assert_that(self.model.get_current_source_number(), equal_to(0))
        assert_that(self.model.get_current_obs_number(), equal_to(0))
        assert_that(observer.on_change_img.call_count, equal_to(3))

    def test_accept_current_item(self):
        workunit = self.model.get_current_workunit()

        data = self.model.get_current_data()
        first_item = data.get_sources()[0]
        second_item = data.get_sources()[1]

        assert_that(workunit.is_item_processed(first_item), equal_to(False))
        assert_that(workunit.is_item_processed(second_item), equal_to(False))

        self.model.accept_current_item()

        assert_that(workunit.is_item_processed(first_item), equal_to(True))
        assert_that(workunit.is_item_processed(second_item), equal_to(False))

        self.model.next_item()
        self.model.accept_current_item()

        assert_that(workunit.is_item_processed(first_item), equal_to(True))
        assert_that(workunit.is_item_processed(second_item), equal_to(True))

    def test_reject_current_item(self):
        workunit = self.model.get_current_workunit()

        data = self.model.get_current_data()
        first_item = data.get_sources()[0]
        second_item = data.get_sources()[1]

        assert_that(workunit.is_item_processed(first_item), equal_to(False))
        assert_that(workunit.is_item_processed(second_item), equal_to(False))

        self.model.reject_current_item()

        assert_that(workunit.is_item_processed(first_item), equal_to(True))
        assert_that(workunit.is_item_processed(second_item), equal_to(False))

        self.model.next_item()
        self.model.reject_current_item()

        assert_that(workunit.is_item_processed(first_item), equal_to(True))
        assert_that(workunit.is_item_processed(second_item), equal_to(True))

    def test_receive_all_sources_processed_event_on_final_accept(self):
        observer = Mock()
        events.subscribe(events.NO_AVAILABLE_WORK, observer.on_all_processed)

        total_items = 3
        item = 0
        while item < total_items - 1:
            self.model.accept_current_item()
            assert_that(observer.on_all_processed.call_count, equal_to(0))
            self.model.next_item()
            item += 1

        self.model.accept_current_item()

        # We try to get the next work unit, and find there is none
        self.model.next_item()
        assert_that(observer.on_all_processed.call_count, equal_to(1))

    def test_receive_all_sources_processed_event_on_final_reject(self):
        observer = Mock()
        events.subscribe(events.NO_AVAILABLE_WORK, observer.on_all_processed)

        total_items = 3
        item = 0
        while item < total_items - 1:
            self.model.accept_current_item()
            assert_that(observer.on_all_processed.call_count, equal_to(0))
            self.model.next_item()
            item += 1

        self.model.reject_current_item()

        # we try to get the next work unit, and find there is none
        self.model.next_item()
        assert_that(observer.on_all_processed.call_count, equal_to(1))


class MultipleAstromDataModelTest(GeneralModelTest):
    def _get_workunit_builder(self, parser, progress_manager):
        return CandidatesWorkUnitBuilder(parser, self.context, self.context,
                                         progress_manager)

    def _get_task(self):
        return tasks.CANDS_TASK

    def _get_working_dir(self):
        return self.get_abs_path(MODEL_TEST_DIR_2)

    def get_files_to_keep(self):
        return ["1584431p15.measure3.cands.astrom", "1616681p10.measure3.cands.astrom"]

    def test_meta_data(self):
        assert_that(self.model.get_current_source_number(), equal_to(0))
        assert_that(self.model.get_obs_count(), equal_to(3)) # for the current data only

    def test_next_source(self):
        first_sources = self.model.get_current_data().get_sources()

        assert_that(self.model.get_current_source_number(), equal_to(0))
        assert_that(self.model.get_current_source(), equal_to(first_sources[0]))
        self.model.next_source()
        assert_that(self.model.get_current_source_number(), equal_to(1))
        assert_that(self.model.get_current_source(), equal_to(first_sources[1]))
        self.model.next_source()
        assert_that(self.model.get_current_source_number(), equal_to(2))
        assert_that(self.model.get_current_source(), equal_to(first_sources[2]))
        self.model.next_source()

        # Note we don't move to next file.  It is not loaded yet since we
        # have not processed the current one.
        assert_that(self.model.get_current_source_number(), equal_to(0))

        self.model.accept_current_item()
        self.model.next_item()
        self.model.accept_current_item()
        self.model.next_item()
        self.model.accept_current_item()

        # Moving to next work unit
        self.model.next_item()

        second_sources = self.model.get_current_data().get_sources()

        assert_that(first_sources, is_not(same_instance(second_sources)))

        assert_that(self.model.get_current_source_number(), equal_to(0))
        assert_that(self.model.get_current_source(), equal_to(second_sources[0]))
        self.model.next_source()
        assert_that(self.model.get_current_source_number(), equal_to(1))
        assert_that(self.model.get_current_source(), equal_to(second_sources[1]))
        self.model.next_source()
        assert_that(self.model.get_current_source_number(), equal_to(2))
        assert_that(self.model.get_current_source(), equal_to(second_sources[2]))
        self.model.next_source()

        # Note we only iterate within the active work unit
        assert_that(self.model.get_current_source_number(), equal_to(0))
        assert_that(self.model.get_current_source(), equal_to(second_sources[0]))

    def test_previous_source(self):
        first_sources = self.model.get_current_data().get_sources()

        assert_that(self.model.get_current_source_number(), equal_to(0))
        assert_that(self.model.get_current_source(), equal_to(first_sources[0]))
        self.model.previous_source()

        # Note we don't move to next file.  It is not loaded yet since we
        # have not processed the current one.
        assert_that(self.model.get_current_source_number(), equal_to(2))
        self.model.accept_current_item()
        self.model.previous_source()
        self.model.accept_current_item()
        self.model.previous_source()
        assert_that(self.model.get_current_source_number(), equal_to(0))
        self.model.accept_current_item()

        # Moving to next work unit
        self.model.next_item()

        second_sources = self.model.get_current_data().get_sources()
        assert_that(first_sources, is_not(same_instance(second_sources)))

        assert_that(self.model.get_current_source_number(), equal_to(0))
        assert_that(self.model.get_current_source(), equal_to(second_sources[0]))
        self.model.previous_source()
        assert_that(self.model.get_current_source_number(), equal_to(2))
        assert_that(self.model.get_current_source(), equal_to(second_sources[2]))
        self.model.previous_source()
        assert_that(self.model.get_current_source_number(), equal_to(1))
        assert_that(self.model.get_current_source(), equal_to(second_sources[1]))
        self.model.previous_source()
        assert_that(self.model.get_current_source_number(), equal_to(0))
        assert_that(self.model.get_current_source(), equal_to(second_sources[0]))
        self.model.previous_source()

        # Note we only iterate in the current work unit
        assert_that(self.model.get_current_source_number(), equal_to(2))
        assert_that(self.model.get_current_source(), equal_to(second_sources[2]))

    def test_move_to_first_observation_of_new_workunit(self):
        workunit1 = self.model.get_current_workunit()
        assert_that(self.model.get_current_obs_number(), equal_to(0))

        self.model.next_workunit()

        workunit2 = self.model.get_current_workunit()
        assert_that(workunit2, is_not(same_instance(workunit1)))

        assert_that(self.model.get_current_obs_number(), equal_to(0))

    def test_accept_moves_to_first_observation_of_new_workunit(self):
        workunit1 = self.model.get_current_workunit()
        assert_that(self.model.get_current_source_number(), equal_to(0))
        assert_that(self.model.get_current_obs_number(), equal_to(0))

        self.model.accept_current_item()
        self.model.next_item()
        assert_that(self.model.get_current_source_number(), equal_to(1))
        assert_that(self.model.get_current_obs_number(), equal_to(0))

        self.model.accept_current_item()
        self.model.next_item()
        assert_that(self.model.get_current_source_number(), equal_to(2))
        assert_that(self.model.get_current_obs_number(), equal_to(0))

        self.model.accept_current_item()
        self.model.next_item()

        workunit2 = self.model.get_current_workunit()
        assert_that(workunit2, is_not(same_instance(workunit1)))

        assert_that(self.model.get_current_source_number(), equal_to(0))
        assert_that(self.model.get_current_obs_number(), equal_to(0))


class RealsModelPersistenceTest(GeneralModelTest):
    def setUp(self):
        super(RealsModelPersistenceTest, self).setUp()

        concurrent_directory_manager = LocalDirectoryWorkingContext(self._get_working_dir())
        self.concurrent_progress_manager = LocalProgressManager(concurrent_directory_manager)

    def _get_task(self):
        return tasks.REALS_TASK

    def _get_workunit_builder(self, parser, progress_manager):
        return RealsWorkUnitBuilder(parser, self.context, self.context,
                                    progress_manager)

    def _get_working_dir(self):
        return self.get_abs_path(FRESH_TEST_DIR)

    def get_directory_to_clean(self):
        return self._get_working_dir()

    def get_files_to_keep(self):
        return TEST_FILES

    def test_record_progress_only_after_source_finished(self):
        first_file = self.model.get_current_filename()

        assert_that(self.concurrent_progress_manager.get_processed_indices(first_file),
                    has_length(0))

        self.model.set_current_source_name(TEST_OBJECT_NAME)

        self.model.accept_current_item()
        assert_that(self.concurrent_progress_manager.is_done(first_file),
                    equal_to(False))
        assert_that(self.concurrent_progress_manager.get_processed_indices(first_file),
                    has_length(0))

        self.model.next_item()
        self.model.accept_current_item()
        assert_that(self.concurrent_progress_manager.is_done(first_file),
                    equal_to(False))
        assert_that(self.concurrent_progress_manager.get_processed_indices(first_file),
                    has_length(0))

        self.model.next_item()
        self.model.accept_current_item()
        assert_that(self.concurrent_progress_manager.is_done(first_file),
                    equal_to(False))
        assert_that(self.concurrent_progress_manager.get_processed_indices(first_file),
                    contains_inanyorder(0))

    def test_record_progress_multiple_sources(self):
        first_file = self.model.get_current_filename()

        assert_that(self.concurrent_progress_manager.get_processed_indices(first_file),
                    has_length(0))

        self.model.set_current_source_name(TEST_OBJECT_NAME)

        # source 1 of 3, reading 1 of 3
        self.model.accept_current_item()
        self.model.next_item()

        # source 1 of 3 reading 2 of 3
        self.model.accept_current_item()
        self.model.next_item()

        # source 1 of 3 reading 3 of 3
        self.model.accept_current_item()
        self.model.next_item()
        assert_that(self.concurrent_progress_manager.is_done(first_file),
                    equal_to(False))
        assert_that(self.concurrent_progress_manager.get_processed_indices(first_file),
                    contains_inanyorder(0))

        self.model.set_current_source_name(TEST_OBJECT_NAME)

        # source 2 of 3 reading 1 of 3
        self.model.accept_current_item()
        self.model.next_item()
        assert_that(self.concurrent_progress_manager.get_processed_indices(first_file),
                    contains_inanyorder(0))

        # source 2 of 3 reading 2 of 3
        self.model.accept_current_item()
        self.model.next_item()
        assert_that(self.concurrent_progress_manager.get_processed_indices(first_file),
                    contains_inanyorder(0))

        # source 2 of 3 reading 3 of 3
        self.model.accept_current_item()
        self.model.next_item()
        assert_that(self.concurrent_progress_manager.is_done(first_file),
                    equal_to(False))
        assert_that(self.concurrent_progress_manager.get_processed_indices(first_file),
                    contains_inanyorder(0, 1))

        self.model.set_current_source_name(TEST_OBJECT_NAME)

        # source 3 of 3 reading 1 of 3
        self.model.next_item()
        self.model.accept_current_item()
        assert_that(self.concurrent_progress_manager.get_processed_indices(first_file),
                    contains_inanyorder(0, 1))

        # source 3 of 3 reading 2 of 3
        self.model.next_item()
        self.model.accept_current_item()
        assert_that(self.concurrent_progress_manager.get_processed_indices(first_file),
                    contains_inanyorder(0, 1))

        # source 3 of 3 reading 3 of 3
        self.model.next_item()
        self.model.accept_current_item()
        assert_that(self.concurrent_progress_manager.is_done(first_file),
                    equal_to(True))
        assert_that(self.concurrent_progress_manager.get_processed_indices(first_file),
                    contains_inanyorder(0, 1, 2))

    def test_file_processed_event(self):
        observer = Mock()
        events.subscribe(events.FINISHED_WORKUNIT, observer.on_file_processed)

        workunit = self.model.get_current_workunit()
        accepts_before_next_file = 9

        while accepts_before_next_file > 1:
            self.model.set_current_source_name(TEST_OBJECT_NAME)
            self.model.accept_current_item()
            self.model.next_item()
            assert_that(observer.on_file_processed.call_count, equal_to(0))
            accepts_before_next_file -= 1

        self.model.accept_current_item()
        assert_that(observer.on_file_processed.call_count, equal_to(1))

        # Make sure it was triggered with the right data
        args = observer.on_file_processed.call_args[0]
        assert_that(args, has_length(1))

        msg = args[0]
        assert_that(msg.topic, equal_to(events.FINISHED_WORKUNIT))
        assert_that(msg.data, equal_to(workunit.get_results_file_paths()))

    def test_unlock_on_exit(self):
        current_file = self.model.get_current_filename()

        assert_that(self.progress_manager.owns_lock(current_file), equal_to(True))
        self.model.exit()
        assert_that(self.progress_manager.owns_lock(current_file), equal_to(False))


class RealsModelPersistenceLoadingTest(GeneralModelTest):
    def setUp(self):
        # Have to set this up here because test cases may modify the .PART file
        partfile = os.path.join(self.get_directory_to_clean(), "xxx3.reals.astrom.PART")
        with open(partfile, "w+b") as filehandle:
            filehandle.write("0\n1\n")

        super(RealsModelPersistenceLoadingTest, self).setUp()

    def _get_task(self):
        return tasks.REALS_TASK

    def _get_workunit_builder(self, parser, progress_manager):
        return RealsWorkUnitBuilder(parser, self.context, self.context,
                                    progress_manager)

    def _get_working_dir(self):
        return self.get_abs_path("data/model_persistence_partial")

    def get_directory_to_clean(self):
        return self._get_working_dir()

    def get_files_to_keep(self):
        return ["xxx1.cands.astrom", "xxx3.reals.astrom"]

    def test_load_partially_processed(self):
        self.model.set_current_source_name(TEST_OBJECT_NAME)

        observer = Mock()
        events.subscribe(events.FINISHED_WORKUNIT, observer)

        workunit = self.model.get_current_workunit()
        assert_that(workunit.get_unprocessed_sources(), has_length(1))

        assert_that(self.model.get_current_source_number(), equal_to(2))
        assert_that(self.model.get_current_obs_number(), equal_to(0))
        self.model.reject_current_item()

        self.model.next_item()
        self.model.accept_current_item()

        self.model.next_item()
        self.model.accept_current_item()

        assert_that(observer.call_count, equal_to(1))

        msg = observer.call_args_list[0][0][0]
        assert_that(msg.data, equal_to(workunit.get_results_file_paths()))


class CandidatesModelPersistenceTest(GeneralModelTest):
    def setUp(self):
        super(CandidatesModelPersistenceTest, self).setUp()

        concurrent_directory_manager = LocalDirectoryWorkingContext(self._get_working_dir())
        self.concurrent_progress_manager = LocalProgressManager(concurrent_directory_manager)

    def _get_task(self):
        return tasks.CANDS_TASK

    def _get_workunit_builder(self, parser, progress_manager):
        return CandidatesWorkUnitBuilder(parser, self.context, self.context,
                                         progress_manager)

    def _get_working_dir(self):
        return self.get_abs_path(FRESH_TEST_DIR)

    def get_directory_to_clean(self):
        return self._get_working_dir()

    def get_files_to_keep(self):
        return TEST_FILES

    def test_record_progress_cands_multiple_files(self):
        first_file = self.model.get_current_filename()

        self.model.accept_current_item()
        assert_that(self.concurrent_progress_manager.is_done(first_file),
                    equal_to(False))
        assert_that(self.concurrent_progress_manager.get_processed_indices(first_file),
                    contains_inanyorder(0))

        self.model.next_item()
        self.model.accept_current_item()
        assert_that(self.concurrent_progress_manager.is_done(first_file),
                    equal_to(False))
        assert_that(self.concurrent_progress_manager.get_processed_indices(first_file),
                    contains_inanyorder(0, 1))

        self.model.next_item()
        self.model.accept_current_item()
        assert_that(self.concurrent_progress_manager.is_done(first_file),
                    equal_to(True))
        assert_that(self.concurrent_progress_manager.get_processed_indices(first_file),
                    contains_inanyorder(0, 1, 2))

        self.model.next_item()

        second_file = self.model.get_current_filename()
        assert_that(second_file, is_not(equal_to(first_file)))
        assert_that(self.concurrent_progress_manager.get_processed_indices(second_file),
                    has_length(0))

        self.model.accept_current_item()
        assert_that(self.concurrent_progress_manager.is_done(second_file),
                    equal_to(False))
        assert_that(self.concurrent_progress_manager.get_processed_indices(second_file),
                    contains_inanyorder(0))

        self.model.next_item()
        self.model.accept_current_item()
        assert_that(self.concurrent_progress_manager.is_done(second_file),
                    equal_to(False))
        assert_that(self.concurrent_progress_manager.get_processed_indices(second_file),
                    contains_inanyorder(0, 1))

        self.model.next_item()
        self.model.accept_current_item()
        assert_that(self.concurrent_progress_manager.is_done(second_file),
                    equal_to(True))
        assert_that(self.concurrent_progress_manager.get_processed_indices(second_file),
                    contains_inanyorder(0, 1, 2))


class CandidatesModelPersistenceLoadingTest(GeneralModelTest):
    def create_part_file_with_indices(self, indices):
        str_indices = ""
        for index in indices:
            str_indices += str(index) + "\n"

        # Have to set this up here because test cases may modify the .PART file
        partfile = os.path.join(self.get_directory_to_clean(), "xxx1.cands.astrom.PART")
        with open(partfile, "w+b") as filehandle:
            filehandle.write(str_indices)

    def _get_task(self):
        return tasks.CANDS_TASK

    def _get_workunit_builder(self, parser, progress_manager):
        return CandidatesWorkUnitBuilder(parser, self.context, self.context,
                                         progress_manager)

    def _get_working_dir(self):
        return self.get_abs_path("data/model_persistence_partial")

    def get_directory_to_clean(self):
        return self._get_working_dir()

    def get_files_to_keep(self):
        return ["xxx1.cands.astrom", "xxx3.reals.astrom"]

    def test_load_partially_processed(self):
        self.create_part_file_with_indices([0, 2])
        super(CandidatesModelPersistenceLoadingTest, self).setUp()

        observer = Mock()
        events.subscribe(events.FINISHED_WORKUNIT, observer)

        workunit = self.model.get_current_workunit()
        assert_that(workunit.get_unprocessed_sources(), has_length(1))

        assert_that(self.model.get_current_source_number(), equal_to(1))
        assert_that(self.model.get_current_obs_number(), equal_to(0))

        self.model.accept_current_item()
        assert_that(observer.call_count, equal_to(1))

        msg = observer.call_args_list[0][0][0]
        assert_that(msg.data, equal_to(workunit.get_results_file_paths()))

    def test_load_fast_forward_through_sources(self):
        self.create_part_file_with_indices([0, 1])
        super(CandidatesModelPersistenceLoadingTest, self).setUp()

        assert_that(self.model.get_current_workunit().get_unprocessed_sources(),
                    has_length(1))

        assert_that(self.model.get_current_source_number(), equal_to(2))
        assert_that(self.model.get_current_obs_number(), equal_to(0))


if __name__ == '__main__':
    unittest.main()



__author__ = "David Rusk <drusk@uvic.ca>"

import wx
import wx.lib.inspection

from tests.manual_inspection.context import testutil
from ossos.gui.views.mainframe import MainFrame


def main():
    app = wx.App()
    frame = MainFrame(testutil.Dummy("Controller"))
    frame.Show()
    wx.lib.inspection.InspectionTool().Show()
    app.MainLoop()


if __name__ == "__main__":
    main()



#!/usr/bin/env python

import urllib

def TAPQuery(RAdeg=180.0, DECdeg=0.0, width=1, height=1):
    """Do a query of the CADC Megacam table.  Get all observations insize the box.  Returns a file-like object"""


    QUERY =( """ SELECT """
             """ COORD1(CENTROID(Plane.position_bounds)) AS "RAJ2000", COORD2(CENTROID(Plane.position_bounds)) AS "DEJ2000", Plane.time_bounds_cval1 as "MJDATE" """
             """ FROM """
             """ caom2.Observation as o JOIN caom2.Plane as Plane on o.obsID=Plane.obsID """
             """ WHERE """
             """ o.collection = 'CFHT' """
             """ AND o.instrument_name = 'MegaPrime' """
             """ AND INTERSECTS( BOX('ICRS', {}, {}, {}, {}), Plane.position_bounds ) = 1 """
             """ AND ( o.proposal_id LIKE '%P05') """)
    # """ AND ( o.proposal_id LIKE '%P05' OR o.proposal_id LIKE '%L03' or o.proposal_id LIKE '%L06' or o.proposal_id
    # in ( '06AF33', '06BF98' ) ) """ )

    QUERY = QUERY.format( RAdeg, DECdeg, width, height)

    data={"QUERY": QUERY,
          "REQUEST": "doQuery",
          "LANG": "ADQL",
          "FORMAT": "votable"}
    
    url="http://www.cadc.hia.nrc.gc.ca/tap/sync"
    

    print url, data

    return urllib.urlopen(url,urllib.urlencode(data))



if __name__ == '__main__':
    ra_cen = (37+7)/2.0
    dec_cen = (20+0)/2.0
    width = 20-0
    height = 37-7
    print TAPQuery(ra_cen, dec_cen, width, height).read()



__author__ = "David Rusk <drusk@uvic.ca>"
import logging
import os
import tempfile
import warnings

from gui import logger

warnings.simplefilter("ignore")
from pyraf import iraf
from astropy.io import fits
from astropy.io import ascii


class TaskError(Exception):
    """Base task error"""


def phot(fits_filename, x_in, y_in, aperture=15, sky=20, swidth=10, apcor=0.3,
         maxcount=30000.0, exptime=1.0, zmag=None, extno=0):
    """
    Compute the centroids and magnitudes of a bunch sources  on fits image.

    :rtype : astropy.table.Table
    :param fits_filename: Name of fits image to measure source photometry on.
    :type fits_filename: str
    :param x_in: x location of source to measure
    :type x_in: float, numpy.array
    :param y_in: y location of source to measure
    :type y_in: float, numpy.array
    :param aperture: radius of circular aperture to use.
    :type aperture: float
    :param sky: radius of inner sky annulus
    :type sky: float
    :param swidth: width of the sky annulus
    :type swidth: float
    :param apcor: Aperture correction to take aperture flux to full flux.
    :type apcor: float
    :param maxcount: maximum linearity in the image.
    :type maxcount: float
    :param exptime: exposure time, relative to zmag supplied
    :type exptime: float
    :param zmag: zeropoint magnitude
    :param extno: extension of fits_filename the x/y location refers to.
    """

    if not hasattr(x_in, '__iter__'):
        x_in = [x_in, ]
    if not hasattr(y_in, '__iter__'):
        y_in = [y_in, ]

    if (not os.path.exists(fits_filename) and
            not fits_filename.endswith(".fits")):
        # For convenience, see if we just forgot to provide the extension
        fits_filename += ".fits"

    try:
        input_hdulist = fits.open(fits_filename)
    except Exception as err:
        logger.debug(str(err))
        raise TaskError("Failed to open input image: %s" % err.message)

    # get the filter for this image
    filter_name = input_hdulist[extno].header.get('FILTER', 'DEFAULT')

    # Some nominal CFHT zeropoints that might be useful
    zeropoints = {"I": 25.77,
                  "R": 26.07,
                  "V": 26.07,
                  "B": 25.92,
                  "DEFAULT": 26.0,
                  "g.MP9401": 32.0,
                  'r.MP9601': 31.9,
                  'gri.MP9603': 33.520}

    if zmag is None:
        logger.warning("No zmag supplied to daophot, looking for header or default values.")
        zmag = input_hdulist[extno].header.get('PHOTZP', zeropoints[filter_name])
        logger.warning("Setting zmag to: {}".format(zmag))
        # check for magic 'zeropoint.used' files
        for zpu_file in ["{}.zeropoint.used".format(os.path.splitext(fits_filename)[0]), "zeropoint.used"]:
            if os.access(zpu_file, os.R_OK):
                with open(zpu_file) as zpu_fh:
                    zmag = float(zpu_fh.read())
                    logger.warning("Using file {} to set zmag to: {}".format(zpu_file, zmag))
                    break

    photzp = input_hdulist[extno].header.get('PHOTZP', zeropoints.get(filter_name, zeropoints["DEFAULT"]))
    if zmag != photzp:
        logger.warning(("zmag sent to daophot: ({}) "
                        "doesn't match PHOTZP value in image header: ({})".format(zmag, photzp)))

    # setup IRAF to do the magnitude/centroid measurements
    iraf.set(uparm="./")
    iraf.digiphot()
    iraf.apphot()
    iraf.daophot(_doprint=0)

    iraf.photpars.apertures = aperture
    iraf.photpars.zmag = zmag
    iraf.datapars.datamin = 0
    iraf.datapars.datamax = maxcount
    iraf.datapars.exposur = ""
    iraf.datapars.itime = exptime
    iraf.fitskypars.annulus = sky
    iraf.fitskypars.dannulus = swidth
    iraf.fitskypars.salgorithm = "mode"
    iraf.fitskypars.sloclip = 5.0
    iraf.fitskypars.shiclip = 5.0
    iraf.centerpars.calgori = "centroid"
    iraf.centerpars.cbox = 5.
    iraf.centerpars.cthreshold = 0.
    iraf.centerpars.maxshift = 2.
    iraf.centerpars.clean = 'no'
    iraf.phot.update = 'no'
    iraf.phot.verbose = 'no'
    iraf.phot.verify = 'no'
    iraf.phot.interactive = 'no'

    # Used for passing the input coordinates
    coofile = tempfile.NamedTemporaryFile(suffix=".coo", delete=False)

    for i in range(len(x_in)):
        coofile.write("%f %f \n" % (x_in[i], y_in[i]))
    coofile.flush()
    # Used for receiving the results of the task
    # mag_fd, mag_path = tempfile.mkstemp(suffix=".mag")
    magfile = tempfile.NamedTemporaryFile(suffix=".mag", delete=False)

    # Close the temp files before sending to IRAF due to docstring:
    # "Whether the name can be used to open the file a second time, while
    # the named temporary file is still open, varies across platforms"
    coofile.close()
    magfile.close()
    os.remove(magfile.name)

    iraf.phot(fits_filename+"[{}]".format(extno), coofile.name, magfile.name)
    pdump_out = ascii.read(magfile.name, format='daophot')
    logging.debug("PHOT FILE:\n"+str(pdump_out))
    if not len(pdump_out) > 0:
        mag_content = open(magfile.name).read()
        raise TaskError("photometry failed. {}".format(mag_content))

    # apply the aperture correction
    pdump_out['MAG'] -= apcor

    # Clean up temporary files generated by IRAF
    os.remove(coofile.name)
    os.remove(magfile.name)
    logger.debug("Computed aperture photometry on {} objects in {}".format(len(pdump_out), fits_filename))

    del input_hdulist
    return pdump_out


def phot_mag(*args, **kwargs):
    """Wrapper around phot which only returns the computed magnitude directly."""
    try:
        return phot(*args, **kwargs)
    except IndexError:
        raise TaskError("No photometric records returned for {0}".format(kwargs))



__author__ = "David Rusk <drusk@uvic.ca>"

import unittest

from astropy.io import fits
from hamcrest import assert_that, equal_to
from mock import patch

from tests.base_tests import FileReadingTestCase
from ossos import storage
from ossos.astrom import AstromParser
from ossos.naming import ProvisionalNameGenerator, DryRunNameGenerator


class ProvisionalNameGeneratorTest(FileReadingTestCase):
    def setUp(self):
        self.undertest = ProvisionalNameGenerator()

    def parse_astrom_header(self, filename="data/naming/E+3+0_21.measure3.cands.astrom"):
        return AstromParser().parse(self.get_abs_path(filename)).get_sources()[0].get_reading(
            0).get_observation_header()

    def parse_fits_header(self, filename="data/naming/cutout-1616690p.fits"):
        return fits.open(self.get_abs_path(filename))[0].header

    @patch("ossos.storage.increment_object_counter")
    def test_generate_name(self, increment_object_counter):
        increment_object_counter.return_value = "01"

        astrom_header = self.parse_astrom_header()
        fits_header = self.parse_fits_header()

        assert_that(self.undertest.generate_name(astrom_header, fits_header),
                    equal_to("O13AE01"))

        increment_object_counter.assert_called_once_with(storage.MEASURE3, "O13AE")

    @patch("ossos.storage.increment_object_counter")
    def test_generate_name_object_header_has_epoch(self, increment_object_counter):
        increment_object_counter.return_value = "01"

        astrom_header = self.parse_astrom_header()
        fits_header = self.parse_fits_header()
        fits_header["OBJECT"] = "13A" + fits_header["OBJECT"]

        assert_that(self.undertest.generate_name(astrom_header, fits_header),
                    equal_to("O13AE01"))

    @patch("ossos.storage.increment_object_counter")
    def test_dry_run_generate_name(self, increment_object_counter):
        self.undertest = DryRunNameGenerator()

        increment_object_counter.return_value = "01"

        astrom_header = self.parse_astrom_header()
        fits_header = self.parse_fits_header()

        assert_that(self.undertest.generate_name(astrom_header, fits_header),
                    equal_to("DRY0001"))


if __name__ == '__main__':
    unittest.main()



__author__ = "David Rusk <drusk@uvic.ca>"

import wx

from tests.manual_inspection.context import testutil
from ossos.gui.views.navigation import NavPanel


def main():
    app = wx.App()
    rootframe = wx.Frame(None)

    NavPanel(rootframe, testutil.Dummy("Controller"))

    rootframe.Show()
    app.MainLoop()


if __name__ == "__main__":
    main()



__author__ = "David Rusk <drusk@uvic.ca>"

import errno
import unittest

from mock import Mock

from ossos.downloads.async import DownloadRequest
from ossos.gui.app import ValidationApplication
from ossos.gui.errorhandling import DownloadErrorHandler
from ossos.gui.views.appview import ApplicationView


class VOSpaceErrorHandlerTest(unittest.TestCase):
    def setUp(self):
        app = Mock(spec=ValidationApplication)
        view = Mock(spec=ApplicationView)
        app.get_view.return_value = view

        self.downloadable_item = Mock(spec=DownloadRequest)

        self.error_handler = DownloadErrorHandler(app)
        self.view = view

    def test_handle_certificate_problem(self):
        message = "Your certificate is expired."
        error = OSError(message)
        error.errno = errno.EACCES

        self.error_handler.handle_error(error, self.downloadable_item)

        self.view.show_certificate_dialog.assert_called_once_with(
            self.error_handler, message)

    def test_handle_connection_timeout(self):
        message = "Connection timed out."
        error = IOError(message)
        error.errno = errno.ECONNREFUSED

        self.error_handler.handle_error(error, self.downloadable_item)

        self.view.show_retry_download_dialog.assert_called_once_with(
            self.error_handler, message)


if __name__ == '__main__':
    unittest.main()



__author__ = "David Rusk <drusk@uvic.ca>"

import unittest

from mock import Mock, MagicMock, ANY
from hamcrest import equal_to, assert_that

from ossos.astrom import SourceReading
from ossos.downloads.cutouts.source import SourceCutout
from ossos.gui.autoplay import AutoplayManager
from ossos.gui import config
from ossos.gui.models.validation import ValidationModel
from ossos.gui.views.appview import ApplicationView
from ossos.gui.controllers import (AbstractController, ProcessRealsController,
                                   ProcessTracksController,
                                   ImageLoadingDialogManager)
from ossos.naming import ProvisionalNameGenerator


class AbstractControllerTest(unittest.TestCase):
    def setUp(self):
        self.model = MagicMock(spec=ValidationModel)

        self.adjusted_x = 150
        self.adjusted_y = 250

        self.model.get_current_pixel_source_point.return_value = (self.adjusted_x, self.adjusted_y)

        self.original_x = 100
        self.original_y = 200

        def reset_location():
            self.model.get_current_pixel_source_point.return_value = (self.original_x, self.original_y)

        self.model.reset_current_source_location.side_effect = reset_location

        self.view = Mock(spec=ApplicationView)
        self.controller = AbstractController(self.model, self.view)
        self.autoplay_manager = Mock(spec=AutoplayManager)
        self.controller.autoplay_manager = self.autoplay_manager

    def test_reset_source_location_updates_model(self):
        self.controller.on_reset_source_location()
        assert_that(self.model.reset_current_source_location.call_count,
                    equal_to(1))

    def test_reset_source_location_redraws_circle_at_original_location(self):
        self.controller.on_reset_source_location()
        self.view.refresh_markers.assert_called_once_with()

        # Don't need to redraw whole image
        assert_that(self.view.display.call_count, equal_to(0))

    def test_reset_colormap(self):
        self.controller.on_reset_colormap()
        self.view.reset_colormap.assert_called_once_with()

    def test_toggle_reticule(self):
        self.controller.on_toggle_reticule_key()
        self.view.toggle_reticule.assert_called_once_with()

    def test_enable_disable_autoplay(self):
        self.controller.on_enable_autoplay()
        self.autoplay_manager.start_autoplay.assert_called_once_with()

        self.controller.on_disable_autoplay()
        self.autoplay_manager.stop_autoplay.assert_called_once_with()

    def test_display_current_image(self):
        cutout = Mock(spec=SourceCutout)
        self.model.get_current_cutout.return_value = cutout

        self.controller.display_current_image()
        self.view.display.assert_called_once_with(cutout)
        self.view.update_displayed_data.assert_called_once_with(
            self.model.get_reading_data(), self.model.get_header_data_list()
        )

        self.model.acknowledge_image_displayed.assert_called_once_with()

    def test_stop_waiting_for_item_when_loaded(self):
        image_loading_dialog_manager = Mock(spec=ImageLoadingDialogManager)
        self.controller.image_loading_dialog_manager = image_loading_dialog_manager

        displayable_item = Mock()
        event = Mock()
        event.data = displayable_item

        self.controller.on_image_loaded(event)

        image_loading_dialog_manager.set_item_done.assert_called_once_with(
            displayable_item)

    def test_enable_disable_autosync(self):
        self.controller.on_enable_auto_sync()
        self.model.enable_synchronization.assert_called_once_with()

        self.controller.on_disable_auto_sync()
        self.model.disable_synchronization.assert_called_once_with()


class RealsControllerTest(unittest.TestCase):
    def setUp(self):
        self.model = MagicMock(spec=ValidationModel)
        self.model.get_current_source_observed_magnitude.return_value = 10, 20, 100, 1

        self.view = Mock(spec=ApplicationView)
        self.controller = ProcessRealsController(
            self.model, self.view, Mock(spec=ProvisionalNameGenerator))

    def test_accept_sets_note1_to_hand_adjusted_if_current_source_adjusted(self):
        self.model.is_current_source_adjusted.return_value = True

        self.controller.on_accept()
        self.view.show_accept_source_dialog.assert_called_once_with(
            ANY,
            ANY,
            ANY,
            ANY,
            ANY,
            ANY,
            ANY,
            note1_choices=ANY,
            note2_choices=ANY,
            note1_default=config.read("MPC.NOTE1_HAND_ADJUSTED"),
            note2_default=ANY,
            default_observatory_code=ANY,
            default_comment=ANY,
            phot_failure=ANY)

    def test_accept_doesnt_set_note1_to_hand_adjusted_if_current_source_not_adjusted(self):
        self.model.is_current_source_adjusted.return_value = False

        self.controller.on_accept()
        self.view.show_accept_source_dialog.assert_called_once_with(
            ANY,
            ANY,
            ANY,
            ANY,
            ANY,
            ANY,
            ANY,
            note1_choices=ANY,
            note2_choices=ANY,
            note1_default=None,
            note2_default=ANY,
            default_observatory_code=ANY,
            default_comment=ANY,
            phot_failure=ANY)


class ImageLoadingDialogManagerTest(unittest.TestCase):
    def setUp(self):
        self.view = Mock(spec=ApplicationView)
        self.undertest = ImageLoadingDialogManager(self.view)

    def test_wait_for_source_shows_dialog(self):
        source_reading = Mock(spec=SourceReading)
        assert_that(self.view.show_image_loading_dialog.called, equal_to(False))
        self.undertest.wait_for_item(source_reading)
        self.view.show_image_loading_dialog.assert_called_once_with()

    def test_dialog_hidden_when_all_sources_ready(self):
        source_reading1 = Mock(spec=SourceReading)
        source_reading2 = Mock(spec=SourceReading)

        self.undertest.wait_for_item(source_reading1)
        self.undertest.wait_for_item(source_reading2)

        self.view.show_image_loading_dialog.assert_called_once_with()
        assert_that(self.view.hide_image_loading_dialog.called, equal_to(False))

        self.undertest.set_item_done(source_reading1)
        assert_that(self.view.hide_image_loading_dialog.called, equal_to(False))
        self.undertest.set_item_done(source_reading2)

        self.view.hide_image_loading_dialog.assert_called_once_with()

    def test_set_item_done_that_wasnt_in_wait_list_gets_ignored(self):
        source_reading1 = Mock(spec=SourceReading)
        source_reading2 = Mock(spec=SourceReading)

        self.undertest.wait_for_item(source_reading1)
        self.undertest.set_item_done(source_reading2)

        self.view.show_image_loading_dialog.assert_called_once_with()
        assert_that(self.view.hide_image_loading_dialog.called, equal_to(False))

    def test_wait_on_item_twice_only_requires_setting_done_once(self):
        source_reading1 = Mock(spec=SourceReading)
        self.undertest.wait_for_item(source_reading1)
        self.undertest.wait_for_item(source_reading1)

        self.view.show_image_loading_dialog.assert_called_once_with()

        self.undertest.set_item_done(source_reading1)
        self.view.hide_image_loading_dialog.assert_called_once_with()


if __name__ == '__main__':
    unittest.main()



__author__ = "David Rusk <drusk@uvic.ca>"

import unittest

from hamcrest import assert_that, close_to
from ossos.downloads.cutouts.focus import SingletFocusCalculator, TripletFocusCalculator

from tests.base_tests import FileReadingTestCase
from ossos.astrom import AstromParser


def assert_tuples_almost_equal(actual, expected, delta=0.0000001):
    assert_that(actual[0], close_to(expected[0], delta))
    assert_that(actual[1], close_to(expected[1], delta))


class SingletFocusCalculatorTest(FileReadingTestCase):
    def setUp(self):
        astrom_data = AstromParser().parse(
            self.get_abs_path("data/1616681p22.measure3.cands.astrom"))
        self.source = astrom_data.get_sources()[0]
        self.reading0 = self.source.get_reading(0)
        self.reading1 = self.source.get_reading(1)
        self.reading2 = self.source.get_reading(2)

        self.undertest = SingletFocusCalculator(self.source)

    def test_get_focal_point_first_reading(self):
        assert_tuples_almost_equal(
            self.undertest.calculate_focus(self.reading0),
            (583.42, 408.46))

    def test_get_focal_point_second_reading(self):
        assert_tuples_almost_equal(
            self.undertest.calculate_focus(self.reading1),
            (586.18, 408.63))

    def test_get_focal_point_third_reading(self):
        assert_tuples_almost_equal(
            self.undertest.calculate_focus(self.reading2),
            (587.80, 407.98))


class TripletFocusCalculatorTest(FileReadingTestCase):
    def setUp(self):
        astrom_data = AstromParser().parse(
            self.get_abs_path("data/1616681p22.measure3.cands.astrom"))
        self.source = astrom_data.get_sources()[0]
        self.reading0 = self.source.get_reading(0)
        self.reading1 = self.source.get_reading(1)
        self.reading2 = self.source.get_reading(2)

        self.undertest = TripletFocusCalculator(self.source)

    def test_calculate_focal_points(self):
        assert_tuples_almost_equal(
            self.undertest.calculate_focus(self.reading0, 0),
            (560.06, 406.51))

        assert_tuples_almost_equal(
            self.undertest.calculate_focus(self.reading1, 0),
            (562.82, 406.68))

        assert_tuples_almost_equal(
            self.undertest.calculate_focus(self.reading2, 0),
            (564.44, 406.03))

        assert_tuples_almost_equal(
            self.undertest.calculate_focus(self.reading0, 1),
            (583.42, 408.46))

        assert_tuples_almost_equal(
            self.undertest.calculate_focus(self.reading1, 1),
            (586.18, 408.63))

        assert_tuples_almost_equal(
            self.undertest.calculate_focus(self.reading2, 1),
            (587.80, 407.98))

        assert_tuples_almost_equal(
            self.undertest.calculate_focus(self.reading0, 2),
            (608.48, 407.17))

        assert_tuples_almost_equal(
            self.undertest.calculate_focus(self.reading1, 2),
            (611.24, 407.34))

        assert_tuples_almost_equal(
            self.undertest.calculate_focus(self.reading2, 2),
            (612.86, 406.69))


if __name__ == '__main__':
    unittest.main()



__author__ = "David Rusk <drusk@uvic.ca>"

import unittest

from hamcrest import assert_that, equal_to
from mock import Mock

from ossos import auth


class AuthTest(unittest.TestCase):
    def test_get_cadc_username_trims_keycode(self):
        auth.get_issuer_common_name = Mock(return_value="drusk_bc4")
        assert_that(auth.get_cadc_username(), equal_to("drusk"))

    def test_get_cadc_username_contains_underscores(self):
        auth.get_issuer_common_name = Mock(return_value="d_rusk_bc4")
        assert_that(auth.get_cadc_username(), equal_to("d_rusk"))

    def test_get_cadc_username_no_keycode(self):
        auth.get_issuer_common_name = Mock(return_value="drusk")
        assert_that(auth.get_cadc_username(), equal_to("drusk"))


if __name__ == '__main__':
    unittest.main()



__author__ = "David Rusk <drusk@uvic.ca>"

import unittest

from hamcrest import assert_that, equal_to, close_to

from ossos.downloads.cutouts.calculator import CutoutCalculator, CoordinateConverter


class CutoutCalculatorTest(unittest.TestCase):
    def setUp(self):
        self.imgsize = (2000, 2000)

    def test_calc_cutout_internal(self):
        self.calculator = CutoutCalculator(100, 200)

        (x0, x1, y0, y1), _ = self.calculator.calc_cutout((500, 600), self.imgsize)
        assert_that(x0, equal_to(400))
        assert_that(x1, equal_to(600))
        assert_that(y0, equal_to(550))
        assert_that(y1, equal_to(650))

    def test_calc_cutout_internal_str(self):
        self.calculator = CutoutCalculator(100, 200)

        (x0, x1, y0, y1), _ = self.calculator.calc_cutout((500, 600), self.imgsize)
        assert_that(x0, equal_to(400))
        assert_that(x1, equal_to(600))
        assert_that(y0, equal_to(550))
        assert_that(y1, equal_to(650))

    def test_calc_cutout_internal_str_float(self):
        self.calculator = CutoutCalculator(100, 200)

        (x0, x1, y0, y1), _ = self.calculator.calc_cutout((500.00, 600.00), self.imgsize)
        assert_that(x0, equal_to(400))
        assert_that(x1, equal_to(600))
        assert_that(y0, equal_to(550))
        assert_that(y1, equal_to(650))

    def test_build_cutout_str(self):
        self.calculator = CutoutCalculator(100, 200)

        cutout_str, _ = self.calculator.build_cutout_str(15, (500, 600), self.imgsize)
        assert_that(cutout_str, equal_to("[15][400:600,550:650]"))

    def test_calc_cutout_internal_converter(self):
        self.calculator = CutoutCalculator(100, 200)

        _, converter = self.calculator.calc_cutout((500, 600), self.imgsize)
        assert_that(converter.convert((400, 550)), equal_to((1, 1)))
        assert_that(converter.convert((600, 550)), equal_to((201, 1)))
        assert_that(converter.convert((400, 650)), equal_to((1, 101)))
        assert_that(converter.convert((600, 650)), equal_to((201, 101)))
        assert_that(converter.convert((500, 600)), equal_to((101, 51)))

    def test_build_cutout_str_converter(self):
        self.calculator = CutoutCalculator(100, 200)

        _, converter = self.calculator.build_cutout_str(15, (500, 600), self.imgsize)
        assert_that(converter.convert((400, 550)), equal_to((1, 1)))
        assert_that(converter.convert((600, 550)), equal_to((201, 1)))
        assert_that(converter.convert((400, 650)), equal_to((1, 101)))
        assert_that(converter.convert((600, 650)), equal_to((201, 101)))
        assert_that(converter.convert((500, 600)), equal_to((101, 51)))

    def test_calc_cutout_boundary_x(self):
        self.calculator = CutoutCalculator(200, 200)

        (x0, x1, y0, y1), _ = self.calculator.calc_cutout((50, 400), self.imgsize)
        assert_that(x0, equal_to(1))
        assert_that(x1, equal_to(201))
        assert_that(y0, equal_to(300))
        assert_that(y1, equal_to(500))

    def test_calc_cutout_boundary_y(self):
        self.calculator = CutoutCalculator(200, 200)

        (x0, x1, y0, y1), _ = self.calculator.calc_cutout((400, 50), self.imgsize)
        assert_that(x0, equal_to(300))
        assert_that(x1, equal_to(500))
        assert_that(y0, equal_to(1))
        assert_that(y1, equal_to(201))

    def test_calc_cutout_boundary_x_converter(self):
        self.calculator = CutoutCalculator(200, 200)

        _, converter = self.calculator.build_cutout_str(15, (50, 400), self.imgsize)
        assert_that(converter.convert((50, 400)), equal_to((50, 101)))
        assert_that(converter.convert((1, 300)), equal_to((1, 1)))

    def test_calc_cutout_boundary_x_converter(self):
        self.calculator = CutoutCalculator(200, 200)

        _, converter = self.calculator.build_cutout_str(15, (400, 50), self.imgsize)
        assert_that(converter.convert((400, 50)), equal_to((101, 50)))
        assert_that(converter.convert((300, 1)), equal_to((1, 1)))

    def test_calc_cutout_boundary_xmax(self):
        self.imgsize = (200, 200)
        self.calculator = CutoutCalculator(100, 100)

        (x0, x1, y0, y1), _ = self.calculator.calc_cutout((175, 100), self.imgsize)
        assert_that(x0, equal_to(100))
        assert_that(x1, equal_to(200))
        assert_that(y0, equal_to(50))
        assert_that(y1, equal_to(150))

    def test_calc_cutout_boundary_ymax(self):
        self.imgsize = (200, 200)
        self.calculator = CutoutCalculator(100, 100)

        (x0, x1, y0, y1), _ = self.calculator.calc_cutout((100, 175), self.imgsize)
        assert_that(x0, equal_to(50))
        assert_that(x1, equal_to(150))
        assert_that(y0, equal_to(100))
        assert_that(y1, equal_to(200))

    def test_calc_cutout_boundary_xmax_converter(self):
        self.imgsize = (200, 200)
        self.calculator = CutoutCalculator(100, 100)

        _, converter = self.calculator.calc_cutout((175, 100), self.imgsize)
        assert_that(converter.convert((175, 100)), equal_to((76, 51)))
        assert_that(converter.convert((100, 50)), equal_to((1, 1)))
        assert_that(converter.convert((200, 150)), equal_to((101, 101)))

    def test_calc_cutout_boundary_xmin(self):
        self.imgsize = (200, 200)
        self.calculator = CutoutCalculator(100, 100)

        (x0, x1, y0, y1), _ = self.calculator.calc_cutout((25, 100), self.imgsize)
        assert_that(x0, equal_to(1))
        assert_that(x1, equal_to(101))
        assert_that(y0, equal_to(50))
        assert_that(y1, equal_to(150))

    def test_calc_cutout_boundary_xmin_converter(self):
        self.imgsize = (200, 200)
        self.calculator = CutoutCalculator(100, 100)

        _, converter = self.calculator.calc_cutout((25, 100), self.imgsize)
        assert_that(converter.convert((25, 100)), equal_to((25, 51)))

    def test_calc_cutout_boundary_ymax_converter(self):
        self.imgsize = (200, 200)
        self.calculator = CutoutCalculator(100, 100)

        _, converter = self.calculator.calc_cutout((100, 175), self.imgsize)
        assert_that(converter.convert((100, 175)), equal_to((51, 76)))
        assert_that(converter.convert((50, 100)), equal_to((1, 1)))
        assert_that(converter.convert((150, 200)), equal_to((101, 101)))

    def test_calc_cutout_inverted(self):
        self.calculator = CutoutCalculator(20, 20)

        (x0, x1, y0, y1), _ = self.calculator.calc_cutout((20, 20), (200, 200),
                                                          inverted=True)
        assert_that(x0, equal_to(190))
        assert_that(x1, equal_to(170))
        assert_that(y0, equal_to(190))
        assert_that(y1, equal_to(170))

    def test_calc_cutout_inverted_converter(self):
        self.calculator = CutoutCalculator(20, 20)

        _, converter = self.calculator.calc_cutout((20, 20), (200, 200),
                                                   inverted=True)
        assert_that(converter.convert((10, 10)), equal_to((0, 0)))
        assert_that(converter.convert((30, 10)), equal_to((20, 0)))
        assert_that(converter.convert((10, 30)), equal_to((0, 20)))
        assert_that(converter.convert((30, 30)), equal_to((20, 20)))
        assert_that(converter.convert((20, 20)), equal_to((10, 10)))

    def test_build_cutout_str_inverted(self):
        self.calculator = CutoutCalculator(20, 20)

        cutout_str, _ = self.calculator.build_cutout_str("10", (20, 20), (200, 200),
                                                         inverted=True)
        assert_that(cutout_str, equal_to("[10][190:170,190:170]"))

    def test_cutout_near_edge(self):
        calculator = CutoutCalculator(250, 250)

        (x0, x1, y0, y1), _ = calculator.calc_cutout(
            (1970.17, 4611.65), (2112, 4644), inverted=False)

        assert_that(x0, equal_to(1845))
        assert_that(x1, equal_to(2095))
        assert_that(y0, equal_to(4394))
        assert_that(y1, equal_to(4644))

    def test_converter_near_edge(self):
        calculator = CutoutCalculator(250, 250)

        _, converter = calculator.calc_cutout(
            (1970.17, 4611.65), (2112, 4644), inverted=False)

        x, y = converter.convert((1970.17, 4611.65))

        delta = 0.01
        # note 1-based indexing not 0
        assert_that(x, close_to(126.17, delta))
        assert_that(y, close_to(218.65, delta))

    def test_cutout_over_two_edges(self):
        calculator = CutoutCalculator(250, 250)

        (x0, x1, y0, y1), _ = calculator.calc_cutout(
            (1997.68, 4618.31), (2112, 4644), inverted=False)

        delta = 0.01
        assert_that(x0, close_to(1862, delta))
        assert_that(x1, close_to(2112, delta))
        assert_that(y0, close_to(4394, delta))
        assert_that(y1, close_to(4644, delta))

    def test_inverse_converter(self):
        original_coords_x = 1500
        original_coords_y = 2000

        converter = CoordinateConverter(1000, 500)

        new_coords_x, new_coords_y = converter.convert(
            (original_coords_x, original_coords_y))

        assert_that(new_coords_x, equal_to(500))
        assert_that(new_coords_y, equal_to(1500))

        inverse_converter = converter.get_inverse_converter

        final_x, final_y = inverse_converter.convert(
            (new_coords_x, new_coords_y))

        assert_that(final_x, equal_to(original_coords_x))
        assert_that(final_y, equal_to(original_coords_y))


if __name__ == '__main__':
    unittest.main()



__author__ = "David Rusk <drusk@uvic.ca>"

import unittest

from hamcrest import assert_that, equal_to, contains, has_length

from tests.base_tests import FileReadingTestCase
from ossos.gui import config


class AppConfigTest(FileReadingTestCase):
    def setUp(self):
        reload(config)

    def conffile(self):
        return self.get_abs_path("data/testconf.json")

    def test_read_values(self):
        appconfig = config.AppConfig(configfile=self.conffile())
        assert_that(appconfig.read("testsection1.key1"), equal_to("val1"))
        assert_that(appconfig.read("testsection1.key2"), equal_to(1.2))
        assert_that(appconfig.read("testsection2.s2key1"), equal_to("someval"))

    def test_read_list(self):
        appconfig = config.AppConfig(configfile=self.conffile())
        assert_that(appconfig.read("testsection1.key3"), contains("lval1", "lval2", "lval3"))

    def test_lazy_instantiation(self):
        assert_that(config._configs, has_length(0))
        config.read("testsection1.key1", configfile=self.conffile())
        assert_that(config._configs, has_length(1))
        config.read("testsection1.key2", configfile=self.conffile())
        assert_that(config._configs, has_length(1))

    def test_module_read_function(self):
        configfile = self.conffile()
        assert_that(config.read("testsection1.key2", configfile), equal_to(1.2))


if __name__ == '__main__':
    unittest.main()



__author__ = "David Rusk <drusk@uvic.ca>"

import unittest

from hamcrest import assert_that, instance_of, equal_to, none
from matplotlib.backend_bases import MouseEvent as MPLMouseEvent
from mock import Mock, patch

from ossos.fitsviewer.displayable import ImageSinglet
from ossos.fitsviewer.interaction import InteractionContext, MoveMarkerState, CreateMarkerState, AdjustColormapState


class InteractionTest(unittest.TestCase):
    def setUp(self):
        mainhdu = Mock()
        mainhdu.data.shape = (100, 100)
        self.hdulist = [mainhdu]
        self.figure = Mock()

        with patch.object(ImageSinglet, "_create_axes"):
            self.image = ImageSinglet(self.hdulist, self.figure, [0, 0, 1, 1])

        self.image.axes = Mock()

        self.interaction_context = InteractionContext(self.image)

    def _create_mouse_event(self, x, y, button, inaxes=True):
        event = Mock(spec=MPLMouseEvent)
        event.x = x
        event.xdata = x
        event.y = y
        event.ydata = y
        event.button = button

        if inaxes:
            event.inaxes = self.image.axes
        else:
            event.inaxes = Mock()  # a new, different axes

        return event

    def fire_press_event(self, x, y, button=InteractionContext.MOUSE_BUTTON_LEFT,
                         inaxes=True):
        self.interaction_context.on_press(
            self._create_mouse_event(x, y, button, inaxes))

    def fire_release_event(self, button=InteractionContext.MOUSE_BUTTON_LEFT):
        event = Mock(spec=MPLMouseEvent)
        event.button = button
        self.interaction_context.on_release(event)

    def fire_motion_event(self, x, y, inaxes=True):
        self.interaction_context.on_motion(
            self._create_mouse_event(x, y, inaxes))

    def test_state_click_in_circle(self):
        x = 10
        y = 10
        radius = 5

        self.image.place_marker(x, y, radius)
        self.fire_press_event(x + 2, y + 2)
        assert_that(self.interaction_context.state, instance_of(MoveMarkerState))

    def test_press_release(self):
        x = 10
        y = 10
        radius = 5

        self.image.place_marker(x, y, radius)
        assert_that(not self.interaction_context.state.pressed)
        self.fire_press_event(x + 2, y + 2)
        assert_that(self.interaction_context.state.pressed)
        self.fire_release_event()
        assert_that(not self.interaction_context.state.pressed)

    def test_state_click_outside_marker(self):
        x = 10
        y = 10
        radius = 5

        self.image.place_marker(x, y, radius)
        self.fire_press_event(x + 2, y + 2)
        assert_that(self.interaction_context.state, instance_of(MoveMarkerState))
        self.fire_release_event()
        assert_that(self.interaction_context.state, instance_of(MoveMarkerState))
        self.fire_press_event(x + 6, y + 6)
        assert_that(self.interaction_context.state, instance_of(CreateMarkerState))

    def test_state_right_click(self):
        x = 10
        y = 10

        self.fire_press_event(x, y, button=InteractionContext.MOUSE_BUTTON_LEFT)
        assert_that(self.interaction_context.state, instance_of(CreateMarkerState))
        self.fire_release_event(button=InteractionContext.MOUSE_BUTTON_LEFT)

        self.fire_press_event(x, y, button=InteractionContext.MOUSE_BUTTON_RIGHT)
        assert_that(self.interaction_context.state, instance_of(AdjustColormapState))
        self.fire_release_event(button=InteractionContext.MOUSE_BUTTON_RIGHT)

        self.fire_press_event(x, y, button=InteractionContext.MOUSE_BUTTON_LEFT)
        assert_that(self.interaction_context.state, instance_of(CreateMarkerState))
        self.fire_release_event(button=InteractionContext.MOUSE_BUTTON_LEFT)

    def test_drag_marker(self):
        x0 = 10
        y0 = 10
        radius = 5

        xclick = x0 + 2
        yclick = y0 + 2
        dx = 10
        dy = 5

        self.image.place_marker(x0, y0, radius)
        assert_that(self.interaction_context.get_marker().center, equal_to((x0, y0)))
        self.fire_press_event(xclick, yclick)

        self.fire_motion_event(xclick + dx, yclick + dy)
        assert_that(self.interaction_context.get_marker().center,
                    equal_to((x0 + dx, y0 + dy)))
        assert_that(self.interaction_context.get_marker().radius, equal_to(radius))

    def test_create_marker(self):
        x0 = 10
        y0 = 10
        dx = 10
        dy = 30

        assert_that(self.interaction_context.get_marker(), none())
        self.fire_press_event(x0, y0)
        self.fire_motion_event(x0 + dx, y0 + dy)
        assert_that(self.interaction_context.get_marker().center,
                    equal_to((15, 25)))
        assert_that(self.interaction_context.get_marker().radius, equal_to(15))

    def test_motion_not_pressed(self):
        x = 10
        y = 10
        radius = 5

        self.image.place_marker(x, y, radius)

        self.interaction_context.state = CreateMarkerState(self.interaction_context)
        self.fire_motion_event(x + 2, y + 2)
        assert_that(self.interaction_context.get_marker().center, equal_to((x, y)))
        assert_that(self.interaction_context.get_marker().radius, equal_to(radius))

        self.interaction_context.state = MoveMarkerState(self.interaction_context)
        self.fire_motion_event(x + 2, y + 2)
        assert_that(self.interaction_context.get_marker().center, equal_to((x, y)))
        assert_that(self.interaction_context.get_marker().radius, equal_to(radius))

    def test_click_no_drag_inside_marker(self):
        x = 10
        y = 10
        radius = 5

        self.image.place_marker(x, y, radius)

        click_x = 12
        click_y = 13
        self.fire_press_event(click_x, click_y)
        self.fire_release_event()

        assert_that(self.interaction_context.get_marker().center,
                    equal_to((click_x, click_y)))

    def test_click_no_drag_outside_marker(self):
        x = 10
        y = 10
        radius = 5

        self.image.place_marker(x, y, radius)

        click_x = 20
        click_y = 21
        self.fire_press_event(click_x, click_y)
        self.fire_release_event()

        assert_that(self.interaction_context.get_marker().center,
                    equal_to((click_x, click_y)))

    def test_xy_changed_event_on_click(self):
        handler = Mock()
        self.image.xy_changed.connect(handler)

        self.image.place_marker(10, 10, 5)

        x_click = 20
        y_click = 30
        self.fire_press_event(x_click, y_click)
        self.fire_release_event()

        handler.assert_called_once_with(x_click, y_click)

    def test_xy_changed_event_on_drag(self):
        handler = Mock()
        self.image.xy_changed.connect(handler)

        x0 = 10
        y0 = 10
        radius = 5
        self.image.place_marker(x0, y0, radius)

        xclick = x0 + 2
        yclick = y0 + 2
        dx = 10
        dy = 20
        self.fire_press_event(xclick, yclick)
        self.fire_motion_event(xclick + dx, yclick + dy)

        handler.assert_called_once_with(x0 + dx, y0 + dy)


if __name__ == '__main__':
    unittest.main()
